create table sscAttribute (
	attributeID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.AttributeBean', 
	attributeCode varchar(100) not null unique, 
	attributeName varchar(255), 
	attributeDescription text, 
	attributeIsActive bool not null default false, 
	attributeType varchar(32) not null, 
	attributeUnitCost float8, 
	attributeUnitPrice float8, 
	attributeAltPrice float8, 
	attributeWeight float8, 
	attributeSmallImage varchar(255), 
	attributeLargeImage varchar(255), 
	attributeIsRequired bool not null default false, 
	defaultOptionID int4, 
	attributeOrder int4 not null default 0, 
	primary key (attributeID)
);
create table sscCategory (
	categoryID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.CategoryBean', 
	parentCategory int4, 
	code varchar(100) not null unique, 
	name varchar(255), 
	description text, 
	keywords varchar(255), 
	isActive bool not null default false, 
	isContentPage bool not null default false, 
	categoryTreeID int4 not null default 0, 
	header text, 
	footer text, 
	smallImage varchar(255), 
	mediumImage varchar(255), 
	largeImage varchar(255), 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	categoryOrder int4 not null default 0, 
	primary key (categoryID)
);
create table sscManufacturer (
  manufacturerID int4 not null,
  code varchar(100) not null unique,
  name varchar(255),
  websiteURL varchar(255),
  description TEXT,
  isActive bool not null default false,
  header text,
  footer text,
  smallImage varchar(255),
  largeImage varchar(255),
  isDropShipper bool not null default false,
  address1 varchar(255),
  address2 varchar(255),
  city varchar(100),
  state varchar(100),
  postalCode varchar(100),
  country varchar(100),
  phone varchar(100),
  email varchar(100),
  extra1 varchar(255),
  extra2 varchar(255),
  extra3 varchar(255),
  manufacturerOrder int4 not null default 0,
  class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ManufacturerBean',
  primary key (manufacturerID)
);
create table sscCategorySetting (
	categorySettingID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.CategorySettingBean', 
	categoryID int4, 
	code varchar(100) not null, 
	name varchar(100), 
	description varchar(255), 
	type varchar(100), 
	valueType varchar(16) not null, 
	integerValue int4, 
	smallValue varchar(64), 
	mediumValue varchar(255), 
	textValue text, 
	primary key (categorySettingID),
	unique (categoryID, code)
);
create table sscDiscount (
	discountID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountBean', 
	code varchar(100) not null unique, 
	name varchar(255), 
	description text, 
	isActive bool not null default false, 
	usersAppliesType varchar(32), 
	usersAppliesValue varchar(255), 
	itemsAppliesType varchar(32), 
	itemsAppliesValue varchar(255), 
	cartItemsType varchar(32), 
	cartItemsValue varchar(255), 
	rewardType varchar(32), 
	reward float8, 
	couponCode varchar(64), 
	startDate varchar(100), 
	expirationDate varchar(100), 
	timesUsed int4, 
	maxTimesUsed int4, 
	isTimesPerCustomer bool not null default false, 
	orderItemID int4, 
	discountOrder int4 not null default 0, 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	primary key (discountID)
);
create table sscDiscountRange (
	discountRangeID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountRangeBean', 
	discountID int4, 
	rewardType varchar(32), 
	reward float8, 
	floor float8 not null, 
	ceiling float8 not null, 
	isActive bool not null, 
	primary key (discountRangeID)
);
create table sscOption (
	optionID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.OptionBean', 
	attributeID int4, 
	optionCode varchar(100) not null, 
	optionName varchar(255), 
	optionDescription text, 
	optionIsActive bool not null default false, 
	optionUnitCost float8, 
	optionUnitPrice float8, 
	optionAltPrice float8, 
	optionWeight float8, 
	optionSmallImage varchar(255), 
	optionLargeImage varchar(255), 
	optionOrder int4 not null default 0, 
	primary key (optionID),
	unique (optionCode, attributeID)
);
create table sscProduct (
	productID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ProductBean', 
	code varchar(100) not null unique, 
	seoCode varchar(255) unique, 
	name varchar(255), 
	description text, 
	keywords varchar(255), 
	shortDescription varchar(255), 
	isActive bool not null default false, 
	isTaxed bool not null default false, 
	unitCost float8, 
	unitPrice float8, 
	altPrice float8, 
	weight float8, 
	header text, 
	footer text, 
	smallImage varchar(255), 
	mediumImage varchar(255), 
	largeImage varchar(255), 
	created VARCHAR(100),
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	extra4 varchar(255), 
	extra5 varchar(255), 
	primaryCategoryID int4, 
	productOrder int4 not null default 0, 
	manufacturerID int4, 
	primary key (productID)
);
create table sscProductAttribute (
	productAttributeID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ProductAttributeBean', 
	productID int4, 
	attributeID int4, 
	productAttributeOrder int4 not null default 0, 
	primary key (productAttributeID),
	unique (productID, attributeID)
);
create table sscProductCategory (
	productCategoryID int4 not null default nextval('hibernate_sequence'::text), 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ProductCategoryBean', 
	productID int4 not null, 
	categoryID int4 not null, 
	productCategoryOrder int4 not null default 0, 
	primary key (productCategoryID),
	unique (productID, categoryID)
);
create table sscProductSetting (
	productSettingID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ProductSettingBean', 
	productID int4, 
	code varchar(100) not null, 
	name varchar(100), 
	description varchar(255), 
	type varchar(100), 
	valueType varchar(16) not null, 
	integerValue int4, 
	smallValue varchar(64), 
	mediumValue varchar(255), 
	textValue text, 
	primary key (productSettingID),
	unique (productID, code)
);
create table sscRelatedProduct (
	relatedProductID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.RelatedProductBean', 
	parentProductID int4, 
	productID int4, 
	relatedType varchar(32), 
	relatedProductOrder int4 not null default 0, 
	primary key (relatedProductID),
	unique (parentProductID, productID)
);
create table sscSKU (
	skuID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.SKUBean', 
	productID int4, 
	attributeID1 int4, 
	optionID1 int4, 
	attributeID2 int4, 
	optionID2 int4, 
	attributeID3 int4, 
	optionID3 int4, 
	attributeID4 int4, 
	optionID4 int4, 
	attributeID5 int4, 
	optionID5 int4, 
	skuNumber varchar(128) not null unique, 
	skuName varchar(255), 
	skuMessage varchar(255), 
	trackInventory bool not null default false, 
	inventoryLevel int4 not null, 
	extra1 varchar(255), 
	extra2 varchar(255), 
	extra3 varchar(255), 
	notes text, 
	primary key (skuID),
	unique (productID, attributeID1, optionID1, attributeID2, optionID2, 
  attributeID3, optionID3, attributeID4, optionID4, attributeID5, optionID5)
);

create table sscSKUDiscount (
	skuDiscountID int4 not null, 
	class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.SKUDiscountBean', 
	discountID int4, 
	skuID int4, 
	primary key (skuDiscountID),
	unique (skuID, discountID)
);

CREATE TABLE sscProductReview (
  productReviewID int4 not null,
  productID int4 not null,
  status int4 default 0,
  class varchar(255) not null DEFAULT 'com.softslate.commerce.businessobjects.product.ProductReviewBean',
  nickname VARCHAR(255),
  firstName VARCHAR(255),
  lastName VARCHAR(255),
  email VARCHAR(255),
  age VARCHAR(255),
  city VARCHAR(255),
  state VARCHAR(255),
  review text,
  rating int4,
  created VARCHAR(255),
  imagePath VARCHAR(255),
  ipaddress VARCHAR(255),
  country VARCHAR(255),
  customerID int4,
  primary key (productReviewID)
);

alter table sscCategory add constraint FK6DD211EA38F9586 foreign key (parentCategory) references sscCategory;
alter table sscCategorySetting add constraint FK346A7526B3F9C77 foreign key (categoryID) references sscCategory;
alter table sscDiscountRange add constraint FK26F4245C4B3E083D foreign key (discountID) references sscDiscount;
alter table sscOption add constraint FK8D2F749532056A73 foreign key (attributeID) references sscAttribute;
alter table sscProduct add constraint FK50C664CF5BCC8FF9 foreign key (primaryCategoryID) references sscCategory;
alter table sscProduct add constraint productManfFK foreign key (manufacturerID) references sscManufacturer;
alter table sscProductAttribute add constraint FKDDC65E8D32056A73 foreign key (attributeID) references sscAttribute;
alter table sscProductAttribute add constraint FKDDC65E8D2357AE59 foreign key (productID) references sscProduct;
alter table sscProductCategory add constraint FKD05546ED2357AE59 foreign key (productID) references sscProduct;
alter table sscProductCategory add constraint FKD05546ED6B3F9C77 foreign key (categoryID) references sscCategory;
alter table sscProductSetting add constraint FK32AC0A812357AE59 foreign key (productID) references sscProduct;
alter table sscRelatedProduct add constraint FK31589E642357AE59 foreign key (productID) references sscProduct;
alter table sscRelatedProduct add constraint FK31589E64F35AC4EF foreign key (parentProductID) references sscProduct;
alter table sscSKU add constraint FK140FD2357AE59 foreign key (productID) references sscProduct;
alter table sscSKUDiscount add constraint FK2685CD5EE324C735 foreign key (skuID) references sscSKU on delete cascade;
alter table sscSKUDiscount add constraint FK2685CD5E4B3E083D foreign key (discountID) references sscDiscount;
alter table sscProductReview add constraint FKPRP foreign key (productID) references sscProduct;

CREATE INDEX IDX_sscCategory_name ON sscCategory (name);
CREATE INDEX IDX_sscCategory_parentCategory ON sscCategory (parentCategory);
CREATE INDEX IDX_sscCategory_categoryOrder ON sscCategory (categoryOrder);

CREATE INDEX IDX_sscManufacturer_name ON sscManufacturer (name);
CREATE INDEX IDX_sscManufacturer_manufacturerOrder ON sscManufacturer (manufacturerOrder);

CREATE INDEX IDX_sscProduct_name ON sscProduct (name);
CREATE INDEX IDX_sscProduct_shortDescription ON sscProduct (shortDescription);
CREATE INDEX IDX_sscProduct_primaryCategoryID ON sscProduct (primaryCategoryID);
CREATE INDEX IDX_sscProduct_productOrder ON sscProduct (productOrder);

CREATE INDEX IDX_sscAttribute_code ON sscAttribute (attributeCode);
CREATE INDEX IDX_sscAttribute_name ON sscAttribute (attributeName);
CREATE INDEX IDX_sscAttribute_order ON sscAttribute (attributeOrder);

CREATE INDEX IDX_sscOption_code ON sscOption (optionCode);
CREATE INDEX IDX_sscOption_name ON sscOption (optionName);
CREATE INDEX IDX_sscOption_order ON sscOption (optionOrder);

CREATE INDEX IDX_sscDiscount_name ON sscDiscount (name);
CREATE INDEX IDX_sscDiscount_discountOrder ON sscDiscount (discountOrder);

CREATE INDEX IDX_sscDiscountRange_reward ON sscDiscountRange (reward);

CREATE INDEX IDX_sscProductReview_rating ON sscProductReview (rating);
CREATE INDEX IDX_sscProductReview_nickname ON sscProductReview (nickname);
