package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * @author David Tobey
 * 
 */
public class CategorySettingsFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CategorySettingsFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		Map properties = PropertyUtils.describe(form);

		// Load settings
		CategoryProcessor categoryProcessor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		try {
			request.setAttribute("categorySettings", categoryProcessor
					.loadSettings(properties));
		} catch (Exception e) {
			log.error(e.toString(), e);
			baseForm.getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.settingsLoadError", e
									.toString()));
			saveErrors(request, (ActionMessages) baseForm.getErrors());
			return mapping.findForward("error");
		}

		// Load parent category
		Category category = categoryProcessor.loadCategoryFromID(PropertyUtils
				.describe(form));
		request.setAttribute("category", category);

		if (request.getAttribute("categorySettingAddEditForm") == null) {
			CategorySettingAddEditForm categorySettingAddEditForm = new CategorySettingAddEditForm();
			categorySettingAddEditForm.initializeProperties(mapping, request);
			categorySettingAddEditForm.setCategoryID(request
					.getParameter("categoryID"));
			categorySettingAddEditForm.setParentName(request
					.getParameter("parentName"));
			request.setAttribute("categorySettingAddEditForm",
					categorySettingAddEditForm);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("productEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Categories",
				"Category Details", "Category Settings" });
		if (category.getIsContentPage()) {
			bcs = Arrays.asList(new String[] { "Home", "Content Pages",
					"Content Page Details", "Content Page Settings" });
		}
		request.setAttribute("breadcrumbs", bcs);

		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		return mapping.findForward("success");
	}
}
