package com.softslate.commerce.administrator.product;

import java.io.InputStream;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Upload an Attachment for the product - a file that is available for download by everyone on the product page, like
 * instruction manuals. Set its size in the ProductSetting for reference.
 * 
 * @author dtobey
 */
public class ProductSettingAddAttachmentAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingAddAttachmentAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		ProductSettingAddEditFormAttachments baseForm = (ProductSettingAddEditFormAttachments) form;
		baseForm.initializeProperties(mapping, request);

		// Upload the file and save it in the fileDownloadSystemPath setting.
		if (baseForm.getFileUpload() != null && baseForm.getFileUpload().getFileSize() > 0) {
			ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
					"actionUtilsImplementer");
			FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory().createObject(
					"fileStorageProcessorImplementer");
			ProductSettingProcessor processor = (ProductSettingProcessor) baseForm.getBusinessObjectFactory()
					.createObject("productSettingProcessorImplementer");

			// Load the imageSystemPath and imagePathURL settings; derive the
			// imageSystemPath from the URL path if blank.
			String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
			String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
			if (!imagePathURL.endsWith("/")) {
				imagePathURL = imagePathURL + "/";
			}
			if (imageSystemPath == null || imageSystemPath.length() == 0) {
				imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
						+ System.getProperty("file.separator");
			}
			if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
				imageSystemPath = imageSystemPath + System.getProperty("file.separator");
			}
			imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
			String filename = actionUtils.makeFilenameNice(baseForm.getFileUpload().getFileName());

			InputStream fileIS = null;
			try {

				// Save the file.
				fileIS = baseForm.getFileUpload().getInputStream();
				fsp.storeFile(fileIS, imageSystemPath + filename, "fileDownload");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName(filename);
				baseForm.setIntegerValue(Integer.toString(baseForm.getFileUpload().getFileSize()));
				baseForm.setTextValue(fsp.urlForFile(imagePathURL, filename, "fileDownload"));
				processor.addProductSetting(PropertyUtils.describe(form));
			} finally {
				if (fileIS != null)
					fileIS.close();
			}
		}

		// Set the success message
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
		saveMessages(baseForm.getRequest(), baseForm.getMessages());

		// Reset the form
		Class classDefinition = baseForm.getClass();
		baseForm = (ProductSettingAddEditFormAttachments) classDefinition.newInstance();
		baseForm.initializeProperties(mapping, request);
		request.setAttribute("productSettingAddEditForm", baseForm);

		return mapping.findForward("success");
	}

}
