package com.softslate.commerce.businessobjects.core;

import java.beans.PropertyDescriptor;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttribute;

/**
 * A utility object containing various methods used by
 * <code>BusinessProcessor</code>s in performing business logic.
 * <p>
 * Each <code>BusinessProcessor</code>s is able to retrieve an instance of
 * <code>BusinessObjectUtils</code> by calling its <code>utils</code> method.
 * 
 * @author David Tobey
 * 
 */
public class BusinessObjectUtilsImpl implements BusinessObjectUtils {

	static Log log = LogFactory.getLog(BusinessObjectUtilsImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.softslate.commerce.businessobjects.core.BusinessObjectUtils#
	 * formatDateTime(java.util.Date)
	 */
	public String formatDateTime(Date dateTime) {
		SimpleDateFormat formatter = new SimpleDateFormat(
				"yyyy-MM-dd HH:mm:ss:SSS", Locale.getDefault());
		return formatter.format(dateTime);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.businessobjects.core.BusinessObjectUtils#parseDateTime
	 * (java.lang.String)
	 */
	public Date parseDateTime(String dateTime) {
		try {
			SimpleDateFormat formatter = new SimpleDateFormat(
					"yyyy-MM-dd HH:mm:ss:SSS", Locale.getDefault());
			return formatter.parse(dateTime);
		} catch (ParseException e) {
			log.error("Error parsing dateTime, returning current time: "
					+ dateTime, e);
			return (new Date());
		}
	}

	public String formatPrice(Double rawPrice, String currencyCode,
			Locale locale) {
		double price = 0;
		if (rawPrice != null) {
			price = rawPrice.doubleValue();
		}
		return formatPrice(price, currencyCode, locale);
	}

	public String formatPrice(double rawPrice, String currencyCode,
			Locale locale) {
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		return nf.format(rawPrice);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.softslate.commerce.businessobjects.core.BusinessObjectUtils#
	 * prepareRequestData(java.util.Map, boolean)
	 */
	public String prepareRequestData(Map params, boolean urlEncode)
			throws Exception {
		StringBuffer sb = new StringBuffer();
		Set keySet = params.keySet();
		Iterator i = keySet.iterator();
		int count = 0;
		while (i.hasNext()) {
			String key = (String) i.next();
			String value = (String) params.get(key);
			if (value != null) {
				if (count > 0) {
					sb.append("&");
				}
				if (urlEncode) {
					sb.append(URLEncoder.encode(key, "UTF-8") + "="
							+ URLEncoder.encode(value, "UTF-8"));
				} else {
					// This is a no-no as per Payflow API v4.
					// value = value.replaceAll("&", "%26");
					// value = value.replaceAll("=", "%3D");
					sb.append(key + "=" + value);
				}
				count++;
			}
		}
		String postData = sb.toString();
		if (log.isDebugEnabled()) {
			String postDataDebugging = postData;
			postDataDebugging = postDataDebugging.replaceAll("cardnum=[^&]*",
					"cardnum=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("cvv2=[^&]*",
					"cvv2=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("CVV2=[^&]*",
					"CVV2=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("pwd=[^&]*",
					"pwd=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("PWD=[^&]*",
					"PWD=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("acct=[^&]*",
					"acct=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll("ACCT=[^&]*",
					"ACCT=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll(
					"x_card_code=[^&]*", "x_card_code=[Removed]");
			postDataDebugging = postDataDebugging.replaceAll(
					"x_card_num=[^&]*", "x_card_num=[Removed]");
			log.debug("Request data: " + postDataDebugging);
		}
		return postData;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.businessobjects.core.BusinessObjectUtils#sendHTTPPost
	 * (java.lang.String, java.lang.String, boolean, Map<String,String>, String)
	 */
	public String sendHTTPPost(String urlString, String postData,
			boolean followRedirect, Map requestProperties, String inputCharSet, Integer timeoutInSeconds)
			throws Exception {
		return this.sendHTTPRequest(urlString, postData, followRedirect, requestProperties, inputCharSet,
				timeoutInSeconds, "POST");
	}

	public String sendHTTPGet(String urlString, String postData, boolean followRedirect, Map requestProperties,
			String inputCharSet, Integer timeoutInSeconds) throws Exception {
		return this.sendHTTPRequest(urlString, postData, followRedirect, requestProperties, inputCharSet,
				timeoutInSeconds, "GET");
	}

	protected String sendHTTPRequest(String urlString, String postData, boolean followRedirect, Map requestProperties,
			String inputCharSet, Integer timeoutInSeconds, String method) throws Exception {
		HttpURLConnection con = null;
		OutputStreamWriter wr = null;
		BufferedReader rd = null;
		StringBuffer response = new StringBuffer();
		try {
			URL url = new URL(urlString);
			con = (HttpURLConnection) url.openConnection();
			if (requestProperties != null) {
				Iterator it = requestProperties.keySet().iterator();
				while (it.hasNext()) {
					String key = (String) it.next();
					String value = (String) requestProperties.get(key);
					con.setRequestProperty(key, value);
				}
			}
			con.setRequestMethod(method);
			con.setInstanceFollowRedirects(followRedirect);
			con.setDoOutput(true);
			con.setDoInput(true);
			con.setUseCaches(false);
			if (timeoutInSeconds != null && timeoutInSeconds > 0) {
				con.setConnectTimeout(timeoutInSeconds * 1000);
			}
			if (postData != null && "POST".equals(method)) {
				wr = new OutputStreamWriter(con.getOutputStream());
				wr.write(postData);
				wr.flush();
			}
			log.info("Response code: " + con.getResponseCode());
			InputStream stream = con.getErrorStream();
			if (stream == null) {
				stream = con.getInputStream();
			}
			if (inputCharSet != null) {
				rd = new BufferedReader(new InputStreamReader(stream, inputCharSet));
			} else {
				rd = new BufferedReader(new InputStreamReader(stream));
			}
			String line;
			while ((line = rd.readLine()) != null) {
				response.append(line);
				response.append("\n");
			}
		} finally {
			// con.disconnect(); "Indicates that other requests to the server are unlikely in the near future."
			if (wr != null) {
				try {
					wr.close();
				} catch (Exception e) {
					// swallow
				}
			}
			if (rd != null) {
				try {
					rd.close();
				} catch (Exception e) {
					// swallow
				}
			}
		}
		String responseString = response.toString();
		if (!followRedirect && (con.getResponseCode() == 301 || responseString.equals(""))) {
			responseString = con.getHeaderField("Location");
		}
		log.debug("Response string: " + responseString);
		return responseString;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.softslate.commerce.businessobjects.core.BusinessObjectUtils#
	 * getMapsFromBeans(java.util.Collection)
	 */
	public Collection getMapsFromBeans(Collection beans) {
		if (log.isInfoEnabled())
			log.info("Start getMapsFromBeans");
		Collection maps = new ArrayList();
		Iterator i = beans.iterator();
		while (i.hasNext()) {
			Object bean = i.next();
			try {
				Map map = new HashMap();
				PropertyDescriptor descriptors[] = PropertyUtils
						.getPropertyDescriptors(bean);
				if (null != descriptors && descriptors.length > 0) {
					for (int j = 0; j < descriptors.length; j++) {
						String pType = descriptors[j].getPropertyType()
								.getName();
						if (!pType.endsWith("Class")
								&& !pType
										.startsWith("com.softslate.commerce.businessobjects")
								&& !pType.equals("java.util.Collection")
								&& !pType.equals("java.util.Map"))
							map.put(descriptors[j].getName(),
									BeanUtils.getProperty(bean, descriptors[j]
											.getName()));
					}
				}
				maps.add(map);
			} catch (Exception e) {
				if (log.isInfoEnabled())
					log.info("Error creating map from bean: " + e.toString());
			}
		}
		if (log.isInfoEnabled())
			log.info("End getMapsFromBeans");
		return maps;
	}

	/**
	 * Returns a DOM Level2 document that represents the given document.
	 */
	public Document parse(InputStream xmlStream, boolean ignoreWhitespace)
			throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setIgnoringElementContentWhitespace(ignoreWhitespace);
		DocumentBuilder db = dbf.newDocumentBuilder();
		// No error handler... this will have to be fixed
		Document doc = db.parse(xmlStream);
		return doc;
	}

	/**
	 * Parses a section of XML and returns the first Element node found
	 */
	public Node parsePartialDocument(String xml)
			throws ParserConfigurationException, SAXException, IOException {
		if (!xml.startsWith("<?xml"))
			xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + xml;

		ByteArrayInputStream bin = new ByteArrayInputStream(xml.getBytes());
		Document doc = parse(bin, true);
		for (Node nodecfg = doc.getFirstChild(); nodecfg != null; nodecfg = nodecfg
				.getNextSibling())
			if (nodecfg.getNodeType() == Node.ELEMENT_NODE)
				return nodecfg;
		return null;
	}

	public Map parseResponseData(String response) throws Exception {
		Map responseMap = new HashMap();
		String[] keyValuePairs = response.split("&");
		for (int i = 0; i < keyValuePairs.length; i++) {
			String keyValuePair = keyValuePairs[i];
			String[] keyValue = keyValuePair.split("=");
			if (keyValue.length > 1) {
				String key = keyValue[0];
				String value = keyValue[1];

				if (log.isDebugEnabled())
					log.debug("Key: " + key + " ,Value: " + value);

				responseMap.put(key, value);
			}
		}
		return responseMap;
	}

	public boolean stringArrayValuesGreaterThan(String[] paramValues, int value) {
		if (paramValues == null) {
			return false;
		}
		try {
			for (int i = 0; i < paramValues.length; i++) {
				try {
					int thisValue = Integer.parseInt(paramValues[i]);
					if (log.isDebugEnabled())
						log.debug("Checking if: " + thisValue
								+ " is greater than: " + value);
					if (thisValue > value) {
						return true;
					}
				} catch (NumberFormatException nfe) {
					if (log.isDebugEnabled())
						log.debug(nfe.getMessage());
				}
			}
		} catch (ArrayIndexOutOfBoundsException aiobe) {
			return false;
		}
		return false;
	}

	public Double roundForMoney(double value) {
		value = value * 100;
		value = Math.round(value);
		value = value / 100;
		return new Double(value);
	}

	public Double roundForMoney(Double value) {
		if (value != null) {
			return roundForMoney(value.doubleValue());
		} else {
			return new Double(0);
		}
	}

	public Map buildOrderItemAttributeMap(OrderItem oi) {
		if (oi == null || oi.getOrderItemAttributes() == null) {
			return null;
		}
		Map attributeMap = new HashMap();
		Iterator i = oi.getOrderItemAttributes().iterator();
		while (i.hasNext()) {
			OrderItemAttribute oia = (OrderItemAttribute) i.next();
			attributeMap.put(oia.getAttributeCode(), oia);
		}
		return attributeMap;
	}

	public boolean doesOrderItemHaveAttributeCode(OrderItem oi,
			String attributeCode) {
		if (oi == null || oi.getOrderItemAttributes() == null) {
			return false;
		}
		Iterator i = oi.getOrderItemAttributes().iterator();
		while (i.hasNext()) {
			OrderItemAttribute oia = (OrderItemAttribute) i.next();
			if (oia.getAttributeCode().equals(attributeCode)) {
				return true;
			}
		}
		return false;
	}

}
