package com.softslate.commerce.businessobjects.email;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.NoSuchProviderException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.sendgrid.XSmtpApiJsonObject;
import com.softslate.commerce.daos.email.EmailDAO;

public class EmailSenderImpl extends BaseBusinessProcessor implements EmailSender {

	static Log log = LogFactory.getLog(EmailSenderImpl.class);

	private Email email = null;

	private String smtpServer = null;

	private String smtpUsername = null;

	private String smtpPassword = null;

	private String smtpPort = null;

	private Properties mailSessionProps = new Properties();

	private Map<String, String> extraHeaders = new HashMap<String, String>();

	private String characterEncoding = null;

	private List<String> attachmentNames = new ArrayList<String>();

	private List<byte[]> attachments = new ArrayList<byte[]>();

	private List<String> mimeTypes = new ArrayList<String>();

	private boolean bypassSendGrid = false;

	private boolean bypassDatabase = false;

	private boolean sendInSeparateThread = false;

	public boolean getBypassDatabase() {
		return bypassDatabase;
	}

	public void setBypassDatabase(boolean bypassDatabase) {
		this.bypassDatabase = bypassDatabase;
	}

	public boolean getSendInSeparateThread() {
		return sendInSeparateThread;
	}

	public void setSendInSeparateThread(boolean sendInSeparateThread) {
		this.sendInSeparateThread = sendInSeparateThread;
	}

	public boolean getBypassSendGrid() {
		return bypassSendGrid;
	}

	public void setBypassSendGrid(boolean bypassSendGrid) {
		this.bypassSendGrid = bypassSendGrid;
	}

	public Email getEmail() {
		return email;
	}

	public void setEmail(Email email) {
		this.email = email;
	}

	public String getSmtpServer() {
		return smtpServer;
	}

	public void setSmtpServer(String smtpServer) {
		this.smtpServer = smtpServer;
	}

	public String getSmtpUsername() {
		return smtpUsername;
	}

	public void setSmtpUsername(String smtpUsername) {
		this.smtpUsername = smtpUsername;
	}

	public String getSmtpPassword() {
		return smtpPassword;
	}

	public void setSmtpPassword(String smtpPassword) {
		this.smtpPassword = smtpPassword;
	}

	public String getSmtpPort() {
		return smtpPort;
	}

	public void setSmtpPort(String smtpPort) {
		this.smtpPort = smtpPort;
	}

	public Properties getMailSessionProps() {
		return mailSessionProps;
	}

	public void setMailSessionProps(Properties mailSessionProps) {
		this.mailSessionProps = mailSessionProps;
	}

	public Map<String, String> getExtraHeaders() {
		return extraHeaders;
	}

	public void setExtraHeaders(Map<String, String> extraHeaders) {
		this.extraHeaders = extraHeaders;
	}

	public String getCharacterEncoding() {
		return characterEncoding;
	}

	public void setCharacterEncoding(String characterEncoding) {
		this.characterEncoding = characterEncoding;
	}

	public List<String> getAttachmentNames() {
		return attachmentNames;
	}

	public void setAttachmentNames(List<String> attachmentNames) {
		this.attachmentNames = attachmentNames;
	}

	public List<byte[]> getAttachments() {
		return attachments;
	}

	public void setAttachments(List<byte[]> attachments) {
		this.attachments = attachments;
	}

	public List<String> getMimeTypes() {
		return mimeTypes;
	}

	public void setMimeTypes(List<String> mimeTypes) {
		this.mimeTypes = mimeTypes;
	}

	public void send() throws Exception {
		if (getSendInSeparateThread()) {
			EmailSendInSeparateThread esst = new EmailSendInSeparateThread();
			new Thread(esst).start();
		} else {
			sendEmail();
		}
	}

	protected void sendEmail() throws Exception {
		if (!skipEmail()) {
			log.info("Preparing to send email: " + email);
			findSmtpServerAndInfo();
			insertEmailBeforeSend();
			defineMailSessionProps();
			Session mailSession = createMailSession();
			MimeMessage msg = new MimeMessage(mailSession);
			msg.setSentDate(new java.util.Date());
			addHeaders(msg);
			addRecipients(msg);
			addFromAddress(msg);
			addSubject(msg);
			addContent(msg);
			int smtpPortInt = defineSmtpPort();
			log.info("Sending email: " + email);
			boolean sendSuccess = sendIt(mailSession, smtpPortInt, msg);
			updateEmailAfterSend(sendSuccess);
		}
	}

	protected boolean skipEmail() throws Exception {
		boolean ret = false;

		// Opted out?
		if (StringUtils.isNotBlank(getEmail().getOptOutType())) {
			EmailOptOutProcessor emailOptOutProcessor = getBusinessObjectFactory().createObject(
					EmailOptOutProcessor.class);
			Integer optOutCount = emailOptOutProcessor.optOutsCountByEmailAndType(getEmail().getOptOutType(),
					getEmail().getToEmails());
			if (optOutCount != null && optOutCount.intValue() > 0) {
				if (log.isInfoEnabled())
					log.info("Skipping email because found in opt out list type: " + getEmail().getOptOutType()
							+ " email: "
							+ getEmail());
				ret = true;
			}
		}

		// Sent this previously?
		if (getEmail().getDontSendIfSameEmailTypeAlreadySent() && StringUtils.isNotBlank(getEmail().getType())) {
			Integer orderID = getEmail().getOrderID();
			if (orderID == null && getEmail().getOrder() != null) {
				orderID = getEmail().getOrder().getOrderID();
			}
			if (orderID != null) {
				EmailProcessor emailProcessor = getBusinessObjectFactory().createObject(EmailProcessor.class);
				Integer existingCount = emailProcessor.emailsCountByTypeAndOrderID(getEmail().getType(),
						orderID);
				if (existingCount != null && existingCount.intValue() > 0) {
					if (log.isInfoEnabled())
						log.info("Skipping email because one of type " + getEmail().getType()
								+ " was previously sent for order id : " + orderID);
					ret = true;
				}
			}
		}
		return ret;
	}

	protected void defineMailSessionProps() {
		getMailSessionProps().put("mail.smtp.host", getSmtpServer());
		if (getSmtpPort() != null && getSmtpPort().equals("465")) {
			getMailSessionProps().put("mail.smtp.starttls.enable", "true");
			getMailSessionProps().put("mail.smtp.socketFactory.port", smtpPort);
			getMailSessionProps().put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
			getMailSessionProps().put("mail.smtp.socketFactory.fallback", "false");
			getMailSessionProps().put("mail.smtp.ssl.protocols", "TLSv1.2");
		}
		if (getSmtpPort() != null && getSmtpPort().equals("587")) {
			getMailSessionProps().put("mail.smtp.port", getSmtpPort());
			getMailSessionProps().put("mail.smtp.starttls.enable", "true");
			getMailSessionProps().put("mail.smtp.ssl.trust", getSmtpServer());
			getMailSessionProps().put("mail.smtp.ssl.protocols", "TLSv1.2");
		}
		if (StringUtils.isNotBlank(getSmtpUsername()) && StringUtils.isNotBlank(getSmtpPassword())) {
			getMailSessionProps().put("mail.smtp.auth", "true");
		}
	}

	protected Session createMailSession() {
		Session mailSession = Session.getInstance(getMailSessionProps(), null);
		mailSession.setDebug(false);
		return mailSession;
	}

	protected void findSmtpServerAndInfo() {
		boolean useSendGrid = false;
		if ("1".equals((String) getSettings().getValue("useSendGrid"))) {
			useSendGrid = true;
		}
		if (getBypassSendGrid()) {
			useSendGrid = false;
		}

		// Default Values
		String smtpServer = (String) getSettings().getValue("storeSMTPServer");
		String smtpUsername = (String) getSettings().getValue("storeSMTPUsername");
		String smtpPassword = (String) getSettings().getValue("storeSMTPPassword");
		String smtpPort = (String) getSettings().getValue("storeSMTPPort");

		if (useSendGrid) {
			smtpServer = (String) getSettings().getValue("sendgridSMTPServer");
			smtpUsername = (String) getSettings().getValue("sendgridSMTPUsername");
			smtpPassword = (String) getSettings().getValue("sendgridSMTPPassword");
			smtpPort = (String) getSettings().getValue("sendgridSMTPPort");
		}

		// If the smtp server was given, use those values otherwise use settings
		if (StringUtils.isBlank(getSmtpServer())) {
			setSmtpServer(smtpServer);
			setSmtpUsername(smtpUsername);
			setSmtpPassword(smtpPassword);
			setSmtpPort(smtpPort);
		}

		if (useSendGrid) {
			Map<String, Object> uniqueMailArgs = new HashMap<String, Object>();
			if (getEmail().getOrder() != null) {
				uniqueMailArgs.put("orderID", getEmail().getOrder().getOrderID());
			}
			if (getEmail().getCustomer() != null) {
				uniqueMailArgs.put("customerID", getEmail().getCustomer().getCustomerID());
			}
			if (getEmail().getTemplate() != null) {
				uniqueMailArgs.put("templateID", getEmail().getTemplate().getTemplateID());
			}
			uniqueMailArgs.put("emailID", email.getEmailID());

			Map<String, Object> customArgs = new HashMap<String, Object>();
			customArgs.put("unique_args", uniqueMailArgs);

			XSmtpApiJsonObject smtpApiJsonObject = new XSmtpApiJsonObject();
			smtpApiJsonObject.to = getEmail().getToEmails().trim().split("\\s*,\\s*");
			if (StringUtils.isNotBlank(getEmail().getBccEmails())) {
				smtpApiJsonObject.to = (String[]) ArrayUtils.addAll(getEmail().getBccEmails().split("\\s*,\\s*"),
						smtpApiJsonObject.to);
			}
			if (StringUtils.isNotBlank(getEmail().getCcEmails())) {
				smtpApiJsonObject.to = (String[]) ArrayUtils.addAll(getEmail().getCcEmails().split("\\s*,\\s*"),
						smtpApiJsonObject.to);
			}
			smtpApiJsonObject.category = (String[]) customArgs.get("category");
			smtpApiJsonObject.unique_args = (Map<String, Object>) customArgs.get("unique_args");
			getExtraHeaders().put("X-SMTPAPI", smtpApiJsonObject.asJson());
		}
		log.info("SMTP server: " + getSmtpServer() + ", SMTP port: " + getSmtpPort());
	}

	protected boolean insertEmailBeforeSend() throws Exception {
		if (!getBypassDatabase() && getDaoFactory() != null && getSettings() != null
				&& getSettings().getValue("recordEmailsInDatabase") != null
				&& getSettings().getValue("recordEmailsInDatabase").equals("1")) {
			log.info("Inserting email in database before sending: " + email);
			try {
				EmailDAO emailDAO =  getDaoFactory().createDAO(EmailDAO.class);
				getEmail().setStatus("Unattempted");
				emailDAO.setEmail(getEmail());
				emailDAO.insertEmail(true);
				// Insert method evicts our object and pulls in the new
				// connected one with the ID
				setEmail(emailDAO.getEmail());
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Cant send email, error inserting email into db.", ex);
				}
				return false;
			}
		} else {
			return false;
		}
		return true;
	}

	protected void addHeaders(MimeMessage msg) throws Exception {
		if (getExtraHeaders() != null) {
			Iterator<Map.Entry<String, String>> i = getExtraHeaders().entrySet().iterator();
			while (i.hasNext()) {
				Map.Entry<String, String> pairs = (Map.Entry<String, String>) i.next();
				msg.addHeader(pairs.getKey(), pairs.getValue());
			}
		}
	}

	protected void addRecipients(MimeMessage msg) throws Exception {

		// Set the to field
		if (getEmail().getToEmails() != null && getEmail().getToEmails().length() > 0) {
			String[] toList = getEmail().getToEmails().split(",");
			InternetAddress[] toAddresses = new InternetAddress[toList.length];
			for (int x = 0; x < toList.length; x++) {
				toAddresses[x] = new InternetAddress(toList[x]);
			}
			msg.setRecipients(Message.RecipientType.TO, toAddresses);
		} else {
			log.warn("To emails is empty.");
		}

		// Set the cc field
		if (getEmail().getCcEmails() != null && getEmail().getCcEmails().length() > 0) {
			String[] ccList = getEmail().getCcEmails().split(",");
			InternetAddress[] ccAddresses = new InternetAddress[ccList.length];
			for (int x = 0; x < ccList.length; x++) {
				ccAddresses[x] = new InternetAddress(ccList[x]);
			}
			msg.setRecipients(Message.RecipientType.CC, ccAddresses);
		}

		// Set the bcc field
		if (getEmail().getBccEmails() != null && getEmail().getBccEmails().length() > 0) {
			String[] bccList = getEmail().getBccEmails().split(",");
			InternetAddress[] bccAddresses = new InternetAddress[bccList.length];
			for (int x = 0; x < bccList.length; x++) {
				bccAddresses[x] = new InternetAddress(bccList[x]);
			}
			msg.setRecipients(Message.RecipientType.BCC, bccAddresses);
		}

	}

	protected void addFromAddress(MimeMessage msg) throws Exception {
		String storeEmailAddress = (String) getSettings().getValue("storeEmailAddress");
		if (StringUtils.isBlank(getEmail().getFromEmail())) {
			getEmail().setFromEmail(storeEmailAddress);
			}
		InternetAddress fromAddress = new InternetAddress(getEmail().getFromEmail());
		msg.setFrom(fromAddress);
	}

	protected void addSubject(MimeMessage msg) throws MessagingException {
		if (getCharacterEncoding() != null) {
			// Use defined charset if contains non US-ASCII
			msg.setSubject(getEmail().getSubject(), getCharacterEncoding());
		} else {
			// User platform default charset if contains non US-ASCII
			msg.setSubject(getEmail().getSubject());
		}
	}

	protected void addContent(MimeMessage msg) throws MessagingException {
		MimeBodyPart wrap = new MimeBodyPart();
		MimeMultipart altMultipart = new MimeMultipart("alternative");
		wrap.setContent(altMultipart);
		MimeMultipart content = new MimeMultipart("related");
		msg.setContent(content);
		content.addBodyPart(wrap);
		MimeBodyPart messageBodyPart;

		// Create your text message part
		if (StringUtils.isNotBlank(getEmail().getBodyText())) {
			messageBodyPart = new MimeBodyPart();
			if (getCharacterEncoding() != null && getCharacterEncoding().length() > 0) {
				messageBodyPart.setText(getEmail().getBodyText(), getCharacterEncoding());
			} else {
				messageBodyPart.setText(getEmail().getBodyText());
			}
			// Add the text part to the multipart
			altMultipart.addBodyPart(messageBodyPart);
		}

		// Create the html part
		messageBodyPart = new MimeBodyPart();
		if (StringUtils.isNotBlank(getEmail().getBodyHtml())) {
			if (getCharacterEncoding() != null && getCharacterEncoding().length() > 0) {
				messageBodyPart.setContent(getEmail().getBodyHtml(), "text/html; charset=\"" + getCharacterEncoding()
						+ "\"");
			} else {
				messageBodyPart.setContent(getEmail().getBodyHtml(), "text/html");
			}
		}
		// Add html part to multi part
		altMultipart.addBodyPart(messageBodyPart);

		// Add one or more attachments
		if (getAttachments() != null && !getAttachments().isEmpty()) {
			for (int i = 0; i < getAttachmentNames().size(); i++) {
				String attachmentName = getAttachmentNames().get(i);
				String mimeType = getMimeTypes().get(i);
				byte[] attachment = getAttachments().get(i);
				MimeBodyPart att = new MimeBodyPart();
				att.setFileName(attachmentName);
				DataSource src = new ByteArrayDataSource(attachment, mimeType);
				att.setDataHandler(new DataHandler(src));
				content.addBodyPart(att);
			}
		}
	}

	protected int defineSmtpPort() {
		int smtpPortInt = 0;
		if (getSmtpPort() != null) {
			try {
				smtpPortInt = Integer.parseInt(getSmtpPort());
			} catch (Exception e) {
				if (log.isInfoEnabled()) {
					log.info("SMTP port setting cannot be parsed " + "into an integer: ", e);
				}
			}
		}
		return smtpPortInt;
	}

	protected boolean sendIt(Session mailSession, int smtpPortInt, MimeMessage msg) {
		boolean sendSuccess = false;
		log.info("Sending email using SMTP server: " + getSmtpServer() + ", SMTP port: " + smtpPortInt
				+ ", SMTP user name: " + getSmtpUsername());
		if (StringUtils.isNotBlank(getSmtpUsername()) && StringUtils.isNotBlank(getSmtpPassword())) {
			try {
				Transport tr = mailSession.getTransport("smtp");
				if (smtpPortInt > 0) {
					tr.connect(getSmtpServer(), smtpPortInt, getSmtpUsername(), getSmtpPassword());
				} else {
					tr.connect(getSmtpServer(), getSmtpUsername(), getSmtpPassword());
				}
				// Send the message
				msg.saveChanges();
				tr.sendMessage(msg, msg.getAllRecipients());
				tr.close();
				sendSuccess = true;
			} catch (NoSuchProviderException nspex) {
				getEmail().setStatus("Send-Error");
				getEmail().setErrorMessage(nspex.getMessage());
				if (log.isErrorEnabled()) {
					log.error("Error opening transport, no such provider, can't send.", nspex);
				}
				sendSuccess = false;
			} catch (MessagingException mex) {
				getEmail().setStatus("Send-Error");
				getEmail().setErrorMessage(mex.getMessage());
				if (log.isErrorEnabled()) {
					log.error("Error sending email.", mex);
				}
				sendSuccess = false;
			}

		} else {
			// Send the message without authentication
			try {
				Transport.send(msg);
				sendSuccess = true;
			} catch (MessagingException mex) {
				getEmail().setStatus("Send-Error");
				getEmail().setErrorMessage(mex.getMessage());
				if (log.isErrorEnabled()) {
					log.error("Error sending email.", mex);
				}
				sendSuccess = false;
			}
		}
		return sendSuccess;
	}

	protected boolean updateEmailAfterSend(boolean successful) throws Exception {
		if (!getBypassDatabase() && getDaoFactory() != null && getSettings() != null
				&& getSettings().getValue("recordEmailsInDatabase") != null
				&& getSettings().getValue("recordEmailsInDatabase").equals("1")) {
			log.info("Updating email in database after sending: " + email);
			try {
				getEmail().setSmtpServer(getSmtpServer());
				EmailDAO emailDAO =  getDaoFactory().createDAO(EmailDAO.class);
				getEmail().setLastActionTime(emailDAO.formatDateTime(new java.util.Date()));
				if (successful) {
					getEmail().setStatus("Sent");
					getEmail().setSendTime(emailDAO.formatDateTime(new java.util.Date()));
				}
				emailDAO.setEmail(getEmail());
				emailDAO.updateEmail(true);
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Error updating email in db after sending.", ex);
				}
				return false;
			}
		} else {
			return false;
		}
		return true;

	}

	protected class EmailSendInSeparateThread implements Runnable {
		public void run() {
			try {
				sendEmail();
			} catch (Exception e) {
				log.error("Exception sending email in separate thread: ", e);
			}
		}
	}

}
