package com.softslate.commerce.businessobjects.events;

import java.lang.reflect.Method;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;

public class EventBusRegistrarImpl extends BaseBusinessProcessor implements EventBusRegistrar {

	static Log log = LogFactory.getLog(EventBusRegistrarImpl.class);

	public void initializeEventBus(EventBus eventBus) {
		for (String componentKey : getAppComponents().stringPropertyNames()) {
			String componentValue = getAppComponents().getProperty(componentKey);
			Class concreteClass = null;
			try {
				concreteClass = Class.forName(componentValue);
				if (classHasMethodWithSubscribeAnnotation(concreteClass)) {
					log.info("Registering class with the event bus: " + componentValue);
					eventBus.register(getBusinessObjectFactory().createObjectFromClassDefinition(concreteClass));
				}
			} catch (ClassNotFoundException cnfe) {
				log.warn("Skipping EventBus registration for concrete class as it can't be found "
						+ "(normal for Enterprise Edition classes when running the Community Edition): "
						+ componentValue);
			} catch (Exception e) {
				log.warn("Error during EventBus registration for concrete class " + componentValue + ": ", e);
			}
		}
	}

	protected boolean classHasMethodWithSubscribeAnnotation(Class concreteClass) {
		boolean ret = false;
		Class subscribeAnnotation = Subscribe.class;
		for (Method meth : concreteClass.getMethods()) {
			if (meth.getAnnotation(subscribeAnnotation) != null) {
				ret = true;
				break;
			}
		}
		return ret;
	}
}
