package com.softslate.commerce.businessobjects.order;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing an order setting within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderSetting</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderSettingImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderSettingImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderSettingBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudOrderSetting")
public interface OrderSetting extends BusinessObject {
	abstract int getOrderSettingID();

	abstract void setOrderSettingID(int orderSettingID);

	abstract Order getOrder();

	abstract void setOrder(Order order);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract Integer getIntegerValue();

	abstract void setIntegerValue(Integer integerValue);

	abstract String getMediumValue();

	abstract void setMediumValue(String mediumValue);

	abstract String getName();

	abstract void setName(String name);

	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract String getSmallValue();

	abstract void setSmallValue(String smallValue);

	abstract String getTextValue();

	abstract void setTextValue(String textValue);

	abstract String getType();

	abstract void setType(String type);

	abstract String getValueType();

	abstract void setValueType(String valueType);

	abstract String getValue();
}
