package com.softslate.commerce.businessobjects.payment.paypoint;

import com.softslate.commerce.daos.payment.PaymentDAO;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Jason McWilliams
 * 
 */
public class PaypointCallbackProcessor extends BaseBusinessProcessor implements
		IPaypointCallbackProcessor {

	static Log log = LogFactory.getLog(PaypointProcessor.class);

	public Map processCallback(Map parameters) throws Exception {

		Map result = new HashMap();
		result.put("resultCode", "1");

		int orderId = 0;

		try {
			orderId = Integer.parseInt(parameters.get("trans_id").toString());
		} catch (Exception ex) {
			if (log.isWarnEnabled())
				log.warn("trans_id is not valid");
		}

		if (log.isInfoEnabled())
			log.info("orderID:" + orderId);

		if (orderId < 1 || getUser().getOrder() == null
				|| orderId != getUser().getOrder().getOrderID()) {
			result.put("resultCode", "2");
			result.put("resultMessage", "Invalid order id.");
			return result;
		}

		// Decide what to do depending on the returned status. We may
		// complete an order, or we may add bad attemtps. We may want
		// to also check for attempts on orders already marked as completed!
		if (getUser().getOrder().getCompleted() != null) {
			result.put("resultCode", "6");
			result.put("resultMessage", "Order already completed.");
			return result;
		}

		String code = (String) parameters.get("code");
		if (code == null) {
			result.put("resultCode", "4");
			result.put("resultMessage", "Invalid result code.");
			return result;
		}

		if (code.equals("A")) {
			addPayment(getUser().getOrder(), parameters, orderId, false);

			// Set the resultCode to 0 to tell the callback action
			// to complete the order
			result.put("resultCode", "0");
			return result;

		} else {

			// Error authorising transaction
			result.put("resultCode", "5");
			if (parameters.get("message") != null
					&& !parameters.get("message").equals("")) {
				result.put("resultMessage", parameters.get("message"));
			} else {
				result.put("resultMessage", "Invalid payment");

			}
		}

		return result;

	}

	protected void addPayment(Order order, Map parameters, int orderId,
			boolean error) throws Exception {
		// Set payment
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		// We wont have credit card name
		payment.setCreditCardNumberDisplay("XXXX");
		payment
				.setProcessorClassName("com.softslate.businessobjects.payment.paypoint.PaypointProcessor");
		if (error) {
			payment.setStatus("Error");
		} else {
			payment.setStatus("Received");
		}
		payment.setValue1("paypoint");
		payment.setValue2(Integer.toString(orderId));
		payment.setValue3((String) parameters.get("amount"));
		payment.setValue4(formatDateTime(new java.util.Date()));
		payment.setValue5((String) parameters.get("auth_code"));
		payment.setValue6((String) parameters.get("code"));
		payment.setValue7((String) parameters.get("cv2avs"));
		payment.setValue8((String) parameters.get("message"));
		payment.setValue9((String) parameters.get("resp_code"));
		payment
				.setValueFieldLabels("order-source,trans_id,amount,timestamp,auth_code,code,cv2avs,message,resp_code");

		payment.setOrder(order);

		if (order.getPayments() == null)
			order.setPayments(new LinkedHashSet());

		order.getPayments().add(payment);

		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);
	}
}
