package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.customer.Customer;

/**
 * Object representing a product review within the system.
 * <p>
 * The database stores product reviews in the <code>sscReview</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>sscProductReviewBean.hbm.xml</code> maps
 * that table's columns to the properties in this object.
 * 
 * @author Tony Zakula
 */
@Audited
@AuditTable("AudProductReview")
public class ProductReviewBean extends BaseBusinessObject implements
		Serializable, ProductReview {

	private static final long serialVersionUID = 1L;

	private Integer productReviewID = null;
	private Integer productID = null;
	private Integer status = null;
	private String firstName = null;
	private String lastName = null;
	private String age = null;
	private String email = null;
	private String city = null;
	private String state = null;
	private String review = null;
	private Integer rating = null;
	private String created = null;
	private String imagePath = null;
	private String ipAddress = null;
	private String country = null;
	private Integer customerID = new Integer(0);
	private Product product = null;
	private Customer customer = null;
	private String nickname = null;

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public Integer getProductReviewID() {
		return productReviewID;
	}

	public void setProductReviewID(Integer productReviewID) {
		this.productReviewID = productReviewID;
	}

	public Integer getProductID() {
		return productID;
	}

	public void setProductID(Integer productID) {
		this.productID = productID;
	}

	public Integer getStatus() {
		return status;
	}

	public void setStatus(Integer status) {
		this.status = status;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getReview() {
		return review;
	}

	public void setReview(String review) {
		this.review = review;
	}

	public Integer getRating() {
		return rating;
	}

	public void setRating(Integer rating) {
		this.rating = rating;
	}

	public String getCreated() {
		return created;
	}

	public void setCreated(String created) {
		this.created = created;
	}

	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		this.imagePath = imagePath;
	}

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getAge() {
		return age;
	}

	public void setAge(String age) {
		this.age = age;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public Integer getCustomerID() {
		return customerID;
	}

	public void setCustomerID(Integer customerID) {
		this.customerID = customerID;
	}

	public Customer getCustomer() {
		return customer;
	}

	public void setCustomer(Customer customer) {
		this.customer = customer;
	}

	public String getNickname() {
		return nickname;
	}

	public void setNickname(String nickname) {
		this.nickname = nickname;
	}

}
