package com.softslate.commerce.businessobjects.shipping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderShippingRule;
import com.softslate.commerce.daos.order.OrderDAO;
import com.softslate.commerce.daos.shipping.ShippingRuleDAO;
import com.softslate.commerce.daos.shipping.ShippingRuleGatewayDAO;

/**
 * Processes business logic having to do with shipping rules. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting discounts.
 * <p>
 * <code>BasicShippingRuleProcessor</code> is the default implementation of
 * the <code>ShippingRuleProcessor</code> interface for the application.
 * 
 * @author Jason McWilliams
 */
public class BasicShippingRuleProcessor extends BaseBusinessProcessor implements
		ShippingRuleProcessor {

	static Log log = LogFactory.getLog(BasicShippingRuleProcessor.class);

	public Map loadShippingRulesAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading shipping rules and count");
		ShippingRuleGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(ShippingRuleGatewayDAO.class);
		return gatewayDAO.loadShippingRulesAndCount(parameters);
	}

	public Map addShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding shipping rule: " + code);
		}
		Map results = new HashMap();
		ShippingRule shippingRule =  getBusinessObjectFactory()
				.createObject(ShippingRule.class);
		BeanUtils.copyProperties(shippingRule, parameters);
		ShippingRuleDAO dao = (ShippingRuleDAO) getDaoFactory().createDAO(
				"shippingRuleDAOImplementer");
		dao.setShippingRule(shippingRule);
		dao.insertShippingRule(true);
		results.put("newRecord", dao.getShippingRule());
		return results;
	}

	public ShippingRule loadShippingRuleFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading shipping rule from id");
		ShippingRule shippingRule =  getBusinessObjectFactory()
				.createObject(ShippingRule.class);
		ShippingRuleDAO dao = (ShippingRuleDAO) getDaoFactory().createDAO(
				"shippingRuleDAOImplementer");
		BeanUtils.copyProperties(shippingRule, parameters);
		dao.setShippingRule(shippingRule);
		dao.loadShippingRuleFromID();
		return dao.getShippingRule();
	}

	public Map editShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing shipping rule: " + code);
		}
		Map results = new HashMap();
		ShippingRule shippingRule = loadShippingRuleFromID(parameters);
		BeanUtils.copyProperties(shippingRule, parameters);
		ShippingRuleDAO dao = (ShippingRuleDAO) getDaoFactory().createDAO(
				"shippingRuleDAOImplementer");
		dao.setShippingRule(shippingRule);
		dao.updateShippingRule(true);
		results.put("updatedRecord", dao.getShippingRule());
		return results;
	}

	public Map editDeleteShippingRules(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple shipping rules");
		ShippingRuleGatewayDAO dao = (ShippingRuleGatewayDAO) getDaoFactory().createDAO(
				"shippingRuleGatewayDAOImplementer");
		Map results = dao.editDeleteShippingRules(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting shipping rule: " + code);
		}
		ShippingRule shippingRule =  getBusinessObjectFactory()
				.createObject(ShippingRule.class);
		BeanUtils.copyProperties(shippingRule, parameters);
		ShippingRuleDAO dao = (ShippingRuleDAO) getDaoFactory().createDAO(
				"shippingRuleDAOImplementer");
		dao.setShippingRule(shippingRule);
		dao.deleteShippingRule(true);
		return;
	}

	/*
	 * Limit Shipping Methods
	 */
	public Collection findAvailableShippingProcessors(
			Collection availableMethods, int orderID) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Finding available shipping processors ");

		}
		if (log.isDebugEnabled())
			log.debug("Get Available Shipping Processors.");

		// Try limiting the shipping methods if we have shippping rules
		// If not, we stick to the default

		if (getUser().getOrder() != null
				&& getUser().getOrder().getOrderShippingRules() != null) {
			processAllowMethods(getUser().getOrder().getOrderShippingRules(),
					availableMethods);
		}

		// Now we have an updated list of available methods we need to find any
		// available discounts and apply them to whatever might be left.
		if (getUser().getOrder() != null
				&& getUser().getOrder().getOrderShippingRules() != null) {
			applyShippingDiscountsToAvailableMethods(getUser().getOrder()
					.getOrderShippingRules(), availableMethods);
		}

		return availableMethods;
	}

	public void applyShippingDiscountsToAvailableMethods(
			Collection orderShippingRules, Collection availableMethods)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Applying discounts to show on shipping methods.");
		/*
		 * We can apply multiple sku discounts that match line items against the
		 * shipping total until it that particular method hits 0
		 * 
		 */
		if (!orderShippingRules.isEmpty()) {
			Iterator i = orderShippingRules.iterator();
			while (i.hasNext()) {
				OrderShippingRule osr = (OrderShippingRule) i.next();
				if (osr.getRuleType().equals("[discount]")) {
					if (osr.getLimitType().equals("[allow]")) {
						applyDiscountToMatchingMethods(availableMethods, osr);
					} else if (osr.getLimitType().equals("[disallow]")) {
						applyDiscountToNotMatchingMethods(availableMethods, osr);
					}
				}
			}
		}
	}

	public void applyDiscount(Map method, OrderShippingRule osr)
			throws Exception {
		Double discountTotal = new Double(0);
		Double priceDouble = (Double) method.get("priceDouble");
		Double originalPriceDouble = (Double) method.get("originalPriceDouble");
		String currencyCode = method.get("currencyCode").toString();
		Collection discounts = (Collection) method.get("discounts");
		if (osr.getDiscountType().equals("[percentage]")) {
			discountTotal = new Double(
					(osr.getDiscount().doubleValue() * originalPriceDouble
							.doubleValue()) / 100);
		} else if (osr.getDiscountType().endsWith("[amount]")) {
			discountTotal = osr.getDiscount();
		}
		if (discountTotal.doubleValue() > 0) {
			if ((priceDouble.doubleValue() - discountTotal.doubleValue()) >= 0) {
				priceDouble = new Double(priceDouble.doubleValue()
						- discountTotal.doubleValue());
				method.put("priceDouble", priceDouble);
			} else {
				priceDouble = new Double(0);
				method.put("priceDouble", priceDouble);
			}
			method.put("price", formatPrice(priceDouble.doubleValue(),
					currencyCode, getUser().getLocale()));
			Map discount = BeanUtils.describe(osr);
			if (discounts != null) {
				discounts.add(discount);
			}
		}
	}

	public void applyDiscountToMatchingMethods(Collection availableMethods,
			OrderShippingRule osr) throws Exception {
		if (osr.getLimitMethods() == null
				|| osr.getLimitMethods().length() == 0) {
			return;
		}
		String[] processorArray = osr.getLimitMethods().split(",");
		for (int pa = 0; pa < processorArray.length; pa++) {
			Map limit = new HashMap();
			String processor = processorArray[pa].trim();
			String[] methodArray = processor.split("\\|");
			if (methodArray.length != 2) {
				continue;
			}
			limit.put("processorClass", methodArray[0].trim());
			limit.put("methodCode", methodArray[1].trim());
			// See if we have this method in our available methods
			Iterator i = availableMethods.iterator();
			while (i.hasNext()) {
				Map method = (Map) i.next();
				// code, name, priceDouble, originalPriceDouble, price,
				// originalPrice, processorClass, processorName
				boolean match = false;
				if (limit.get("processorClass").equals("*")) {
					match = true;
				} else if (limit.get("processorClass").equals(
						method.get("processorClass"))
						&& limit.get("methodCode").equals("*")) {
					match = true;
				} else if (limit.get("processorClass").equals(
						method.get("processorClass"))
						&& limit.get("methodCode").equals(method.get("code"))) {
					match = true;
				}
				if (match) {
					applyDiscount(method, osr);
				}
			}
		}
	}

	public void applyDiscountToNotMatchingMethods(Collection availableMethods,
			OrderShippingRule osr) throws Exception {
		String[] processorArray = osr.getLimitMethods().split(",");
		for (int pa = 0; pa < processorArray.length; pa++) {
			String processor = processorArray[pa].trim();
			Map limit = new HashMap();
			String[] methodArray = processor.split("\\|");
			limit.put("processorClass", methodArray[0].trim());
			limit.put("methodCode", methodArray[1].trim());
			// See if we have this method in our available methods
			Iterator i = availableMethods.iterator();
			while (i.hasNext()) {
				Map method = (Map) i.next();
				// code, name, priceDouble, originalPriceDouble, price,
				// originalPrice, processorClass, processorName
				boolean match = false;
				if (limit.get("processorClass").equals("*")) {
					match = true;
				} else if (limit.get("processorClass").equals(
						method.get("processorClass"))
						&& limit.get("methodCode").equals("*")) {
					match = true;
				} else if (limit.get("processorClass").equals(
						method.get("processorClass"))
						&& limit.get("methodCode").equals(method.get("code"))) {
					match = true;
				}
				if (!match) {
					applyDiscount(method, osr);
				}
			}
		}
	}

	public void processAllowMethods(Collection orderShippingRules,
			Collection availableMethods) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing allowed shipping methods.");
		if (!orderShippingRules.isEmpty()) {
			// Remove all disallowed
			Iterator i = orderShippingRules.iterator();
			while (i.hasNext()) {
				OrderShippingRule osr = (OrderShippingRule) i.next();
				if (osr.getLimitMethods() != null
						&& osr.getLimitMethods().length() > 0
						&& (osr.getRuleType().equals("[package]")
								|| osr.getRuleType().equals("[methodlimit]") || osr
								.getRuleType().equals("[limitByLocation]"))) {
					String[] processorArray = osr.getLimitMethods().split(",");
					for (int pa = 0; pa < processorArray.length; pa++) {
						Map limit = new HashMap();
						String processor = processorArray[pa].trim();
						String[] methodArray = processor.split("\\|");
						if (methodArray.length < 2) {
							continue;
						}
						limit.put("processorClass", methodArray[0].trim());
						limit.put("methodCode", methodArray[1].trim());
						boolean removeMethod = osr.getLimitType().equals(
								"[disallow]")
								&& (!osr.getRuleType().equals(
										"[limitByLocation]") || doesLimitByLocationRuleApply(osr));
						if (removeMethod) {
							if (log.isDebugEnabled())
								log
										.debug("Disallow Found Shipping Method - Rule Class: "
												+ methodArray[0].trim()
												+ " Method: "
												+ methodArray[1].trim());
							removeShippingMethodFromAvailableList(limit,
									availableMethods);
						}
					}
				}
			}
			// Remove all that aren't allowed
			Iterator ai = orderShippingRules.iterator();
			while (ai.hasNext()) {
				OrderShippingRule osr = (OrderShippingRule) ai.next();
				if (osr.getLimitMethods() != null
						&& osr.getLimitMethods().length() > 0
						&& (osr.getRuleType().equals("[package]")
								|| osr.getRuleType().equals("[methodlimit]") || osr
								.getRuleType().equals("[limitByLocation]"))) {
					String[] processorArray = osr.getLimitMethods().split(",");
					Collection hasToBe = new ArrayList();
					for (int pa = 0; pa < processorArray.length; pa++) {
						Map limit = new HashMap();
						String processor = processorArray[pa].trim();
						String[] methodArray = processor.split("\\|");
						if (methodArray.length < 2) {
							continue;
						}
						limit.put("processorClass", methodArray[0].trim());
						limit.put("methodCode", methodArray[1].trim());
						boolean hasToBeAllowed = osr.getLimitType().equals(
								"[allow]")
								&& (!osr.getRuleType().equals(
										"[limitByLocation]") || doesLimitByLocationRuleApply(osr));
						if (hasToBeAllowed) {
							if (log.isDebugEnabled())
								log
										.debug("Allow Found Shipping Method - Rule Class: "
												+ methodArray[0].trim()
												+ " Method: "
												+ methodArray[1].trim());
							hasToBe.add(limit);
						}
					}
					if (!hasToBe.isEmpty()) {
						removeShippingMethodFromAvailableList(hasToBe,
								availableMethods);
					}
				}
			}
		}
	}

	public void removeShippingMethodFromAvailableList(Collection hasToBe,
			Collection availableMethods) throws Exception {
		if (log.isDebugEnabled())
			log
					.debug("Trying to remove everything but our hasToBe shipping list");
		Iterator i = availableMethods.iterator();
		// Loop over our available methods
		while (i.hasNext()) {
			Map availableMethod = (Map) i.next();
			boolean found = false;
			// Make sure this method is in our list of has to be methods
			Iterator ih = hasToBe.iterator();
			while (ih.hasNext()) {
				Map hasTo = (Map) ih.next();
				if (availableMethod.get("processorClass").equals(
						hasTo.get("processorClass"))) {
					if (hasTo.get("methodCode").equals("*")
							|| hasTo.get("methodCode").equals(
									availableMethod.get("code"))) {
						found = true;
						break;
					}
				}
			}
			// Dont remove if the hasToBe list is empty
			if (!found && !hasToBe.isEmpty()) {
				// This available method was not in our list of has to be
				// methods
				i.remove();
			}
		}
	}

	public void removeShippingMethodFromAvailableList(Map removeMethod,
			Collection availableMethods) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Trying to remove method - Rule Class: "
					+ removeMethod.get("processorClass") + " Method: "
					+ removeMethod.get("methodCode"));
		Iterator i = availableMethods.iterator();
		while (i.hasNext()) {
			Map method = (Map) i.next();
			if (removeMethod.get("processorClass").equals(
					method.get("processorClass"))) {
				if (removeMethod.get("methodCode").equals("*")
						|| removeMethod.get("methodCode").equals(
								method.get("code"))) {
					i.remove();
				}
			}
		}
	}

	/*
	 * Process packages stored in orderShippingRules table
	 */

	public Map processOrderComplete(Map parameters) throws Exception {
		// Just call the add items which clears everything out and starts over
		// each time
		return processShippingRules(null);
	}

	public Map processEditItem(Collection newOrderItems) throws Exception {
		return processShippingRules(newOrderItems);
	}

	public Map processAddItems(Collection newOrderItems) throws Exception {
		return processShippingRules(newOrderItems);
	}

	public Map processOnCheckout(Map parameters) throws Exception {
		return processShippingRules(null);
	}

	public Map processShippingRules(Collection newOrderItems) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing shipping rules.");
		if (getUser().getOrder() != null
				&& getUser().getOrder().firstDelivery() != null) {

			// Remove all existing order shipping rules
			processRemoveAllRules();

			// Use the existing order items in the user's cart (the new items
			// should already have been added)
			Collection orderItems = getUser().getOrder().firstDelivery()
					.getOrderItemCollection();
			processSkuShippingRules(orderItems);

			// Process the global rules - this would include method limits based
			// on location
			processGlobalShippingRules();

			// Update the order in case it is detached from the Hibernate
			// session. The transaction is not committed here.
			OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
					"orderDAOImplementer");
			orderDAO.setOrder(getUser().getOrder());
			orderDAO.updateOrder(true);

		}
		return null;
	}

	public Map processGlobalShippingRules() throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing global shipping rules.");

		// Start by processing the method limitations by location
		Collection limitByLocationRules = getLimitByLocationShippingRules();

		// Loop through the limit by location rules
		Iterator j = limitByLocationRules.iterator();
		while (j.hasNext()) {
			ShippingRule rule = (ShippingRule) j.next();

			if (rule.getShippingRuleRanges() != null
					&& !rule.getShippingRuleRanges().isEmpty()) {
				Iterator k = rule.getShippingRuleRanges().iterator();
				while (k.hasNext()) {
					ShippingRuleRange srr = (ShippingRuleRange) k.next();

					// All limit by location rules should be saved with the
					// order because the delivery address check won't be made
					// until
					// the shipping screen.
					if (log.isDebugEnabled())
						log.debug("Creating global shipping rule: "
								+ rule.getCode());
					createGlobalOrderShippingRule(rule, srr);
				}
			}

		}
		return null;
	}

	public Map processSkuShippingRules(Collection orderItems) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Processing SKU shipping rules.");
		if (orderItems == null || orderItems.isEmpty()) {
			return null;
		}

		Collection productIDs = getProductIDs(orderItems);
		Collection attributeIDs = getAttributeIDs(orderItems);
		Collection matchingSkus = loadMatchingSkus(orderItems, productIDs,
				attributeIDs);

		int leftOverQuantity = 0;
		double leftOverWeight = 0;
		double leftOverTotal = 0.00;
		boolean haveSkus = true;

		// If no skus match, we just put in the generic line by skipping most
		// of the processing below
		if (matchingSkus == null || matchingSkus.isEmpty()) {
			if (log.isDebugEnabled())
				log.debug("No matching skus found.");
			haveSkus = false;
		}

		// Loop through the order items.
		Iterator i = orderItems.iterator();
		while (i.hasNext()) {
			OrderItem oi = (OrderItem) i.next();
			int unpackagedQuantity = oi.getQuantity();
			// Keep track of quantity we have packaged
			if (haveSkus) {
				// Find skus matching this order item
				if (log.isDebugEnabled())
					log.debug("Finding Matching Line Item Skus.");
				Collection skuIDs = getSkuIDs(oi, matchingSkus);
				if (skuIDs == null || skuIDs.isEmpty()) {
					leftOverQuantity += unpackagedQuantity;
					// Weight seems to be already added up
					leftOverWeight += oi.getWeight().doubleValue()
							/ oi.getQuantity() * unpackagedQuantity;
					leftOverTotal += oi.getTotal().doubleValue()
							/ oi.getQuantity() * unpackagedQuantity;
					continue;
				}
				// Get shipping rules matching our found skus
				if (log.isDebugEnabled())
					log.debug("Finding matching shipping rules by sku.");
				Collection rules = getShippingRulesBySkuIDs(skuIDs);
				if (rules == null || rules.isEmpty()) {
					if (log.isDebugEnabled())
						log.debug("No rules found.");
					leftOverQuantity += unpackagedQuantity;
					leftOverWeight += oi.getWeight().doubleValue()
							/ oi.getQuantity() * unpackagedQuantity;
					leftOverTotal += oi.getTotal().doubleValue()
							/ oi.getQuantity() * unpackagedQuantity;
					continue;
				}

				// Do the package rules first, keeping track of the left overs
				unpackagedQuantity = processPackages(oi, rules);

				// Now add the possible matching shipping discounts.
				processPossibleDiscounts(oi, rules);

				// Now process the method limit rules
				processMethodLimits(oi, rules);
			}

			// If we have anything left over after the rules, we need to add it
			// to the final generic package
			// This could happen if no rules applied to any line item. We just
			// add up the quantities and the
			// weights for a generic shipping quote later.
			leftOverQuantity += unpackagedQuantity;
			leftOverWeight += oi.getWeight().doubleValue() / oi.getQuantity()
					* unpackagedQuantity;
			leftOverTotal += oi.getTotal().doubleValue() / oi.getQuantity()
					* unpackagedQuantity;

		}
		if (leftOverQuantity > 0) {
			createLeftOverPackage(leftOverQuantity, leftOverWeight,
					leftOverTotal);
		}

		return null;
	}

	public Collection getLimitByLocationShippingRules() throws Exception {
		ShippingRuleGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(ShippingRuleGatewayDAO.class);
		return gatewayDAO.loadLimitByLocationShippingRules();
	}

	public boolean doesLimitByLocationRuleApply(OrderShippingRule osr)
			throws Exception {
		if (getUser().getOrder() == null
				|| getUser().getOrder().firstDelivery() == null) {
			if (log.isDebugEnabled())
				log
						.debug("Order or delivery not defined - rule does not match.");
			return false;
		}
		if (getUser().getOrder().firstDelivery().getCountry() != null
				&& getUser().getOrder().firstDelivery().getCountry().length() > 0
				&& osr.getCountries() != null
				&& osr.getCountries().indexOf(
						getUser().getOrder().firstDelivery().getCountry()) != -1) {
			if (log.isDebugEnabled())
				log.debug("Rule " + osr.getCode() + " matches on country: "
						+ getUser().getOrder().firstDelivery().getCountry());
			return true;
		}
		if (getUser().getOrder().firstDelivery().getState() != null
				&& getUser().getOrder().firstDelivery().getState().length() > 0
				&& osr.getStates() != null
				&& osr.getStates().indexOf(
						getUser().getOrder().firstDelivery().getState()) != -1) {
			if (log.isDebugEnabled())
				log.debug("Rule " + osr.getCode() + " matches on state: "
						+ getUser().getOrder().firstDelivery().getState());
			return true;
		}
		if (log.isDebugEnabled())
			log.debug("Limit by location rule " + osr.getCode()
					+ " does not match.");
		return false;
	}

	public void createGlobalOrderShippingRule(ShippingRule sr,
			ShippingRuleRange srr) throws Exception {
		// Add a shipping rule that applies to the order as a whole
		if (log.isDebugEnabled())
			log.debug("Adding a global order shipping rule: " + sr.getCode());
		OrderShippingRule osr =  getBusinessObjectFactory()
				.createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(osr, sr);
		BeanUtils.copyProperties(osr, srr);
		osr.setOrder(getUser().getOrder());
		if (getUser().getOrder().getOrderShippingRules() == null) {
			getUser().getOrder().setOrderShippingRules(new LinkedHashSet());
		}
		getUser().getOrder().getOrderShippingRules().add(osr);
	}

	public boolean rangeMatch(ShippingRuleRange srr, OrderItem oi)
			throws Exception {

		// Is it active
		if (!srr.getIsActive()) {
			return false;
		}
		// Is it above the floor
		if (srr.getFloor().doubleValue() > oi.getQuantity()) {
			return false;
		}
		// Is it under the ceiling
		if (srr.getCeiling().doubleValue() != 0
				&& srr.getCeiling().doubleValue() <= oi.getQuantity()) {
			return false;
		}

		// We meet the range
		return true;
	}

	public int processPackages(OrderItem oi, Collection rules) throws Exception {

		// Loop through the rules
		Iterator j = rules.iterator();

		// Keep track of the left over quantity that should go into a generic
		// package. The left over quantity is the quantity left over by the
		// shipping
		// rule that packages the most of all the rules in effect.
		int leftOverQuantity = oi.getQuantity();

		// Keep track of which rule this is, if there's more than one rule for
		// the order item. We only add the product weight from the order item
		// for the first rule's packages (and even then only if the range's
		// floor and ceiling are notequal - enforced in the createPackage
		// method).
		// And, we only define the subtotal for the first rule's packages.
		boolean firstRule = true;

		while (j.hasNext()) {
			ShippingRule rule = (ShippingRule) j.next();

			// Should this shipping rule be applied
			if (log.isDebugEnabled())
				log.debug("Checking if rule ok: " + rule.getCode());
			if (!isShippingRuleOk(rule, oi)) {
				continue;
			}

			// Only work on package types
			if (!rule.getRuleType().equals("[package]")) {
				continue;
			}

			// We need to loop through the ranges and see if we have
			// anything that applies. They should be ordered by ceiling
			// in decending order so that we can apply the most items
			// per shipping packages
			int currentQuantity = oi.getQuantity();
			if (rule.getShippingRuleRanges() != null
					&& !rule.getShippingRuleRanges().isEmpty()) {
				Iterator k = rule.getShippingRuleRanges().iterator();
				while (k.hasNext() && currentQuantity > 0) {
					ShippingRuleRange srr = (ShippingRuleRange) k.next();

					// Make sure we have the same or more than the floor
					if (log.isDebugEnabled())
						log.debug("srr.getFloor(): " + srr.getFloor());
					if (currentQuantity >= srr.getFloor().doubleValue()
							&& srr.getIsActive()) {
						if (log.isDebugEnabled())
							log.debug("srr.getFloor(): " + srr.getFloor());

						// Try to add to the package. This is a little
						// tricky. A shipping rule could say that 12
						// fits into a package. If we have 36, 45, or
						// whatever, we need to keep calling the same
						// rule until we no longer are using it.
						// (newQuantity == currentQuantity)
						int newQuantity = -1;
						while (newQuantity != currentQuantity
								&& currentQuantity >= srr.getFloor()
										.doubleValue()) {
							newQuantity = createPackage(rule, srr, oi,
									currentQuantity, firstRule);
							if (newQuantity != currentQuantity) {
								currentQuantity = newQuantity;
								newQuantity = -1;
							}
						}
						firstRule = false;
					}
				}
				if (currentQuantity < leftOverQuantity) {
					leftOverQuantity = currentQuantity;
				}
			}
		}

		return leftOverQuantity;

	}

	public void processPossibleDiscounts(OrderItem oi, Collection rules)
			throws Exception {
		// We add all
		// discounts here no matter what method they may choose. This is
		// so that when we show the methods, we can show discounted
		// prices later. Only 1 discount will be applied in the end
		// because they only choose 1 method. Loop through the skus
		Iterator j2 = rules.iterator();
		// We can ONLY apply 1 shipping rule per SKU!
		while (j2.hasNext()) {
			ShippingRule rule = (ShippingRule) j2.next();

			// Only do discounts this time around
			if (!rule.getRuleType().equals("[discount]")) {
				continue;
			}

			// Should this shipping rule be applied
			if (log.isDebugEnabled())
				log.debug("Checking if discount rule applies: "
						+ rule.getCode());

			if (rule.getShippingRuleRanges() != null
					&& !rule.getShippingRuleRanges().isEmpty()) {
				Iterator k2 = rule.getShippingRuleRanges().iterator();
				while (k2.hasNext()) {
					ShippingRuleRange srr = (ShippingRuleRange) k2.next();
					if (rangeMatch(srr, oi)) {
						// We have an discount match for the orderItem
						createPossibleDiscount(rule, srr, oi);
					}
				}
			}
		}
	}

	public void processMethodLimits(OrderItem oi, Collection rules)
			throws Exception {

		Iterator j2 = rules.iterator();
		// We can ONLY apply 1 shipping rule per SKU!
		while (j2.hasNext()) {
			ShippingRule rule = (ShippingRule) j2.next();

			// Only do method limit rules this time around
			if (!rule.getRuleType().equals("[methodlimit]")) {
				continue;
			}

			// Should this shipping rule be applied
			if (log.isDebugEnabled())
				log.debug("Checking if method limit rule applies: "
						+ rule.getCode());

			if (rule.getShippingRuleRanges() != null
					&& !rule.getShippingRuleRanges().isEmpty()) {
				Iterator k2 = rule.getShippingRuleRanges().iterator();
				while (k2.hasNext()) {
					ShippingRuleRange srr = (ShippingRuleRange) k2.next();
					if (rangeMatch(srr, oi)) {
						// We have an method limit match for the orderItem
						createMethodLimit(rule, srr, oi);
					}
				}
			}
		}
	}

	public void createLeftOverPackage(int quantity, double weight,
			double leftOverTotal) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Adding leftover shipping rule: ");
		OrderShippingRule osr =  getBusinessObjectFactory()
				.createObject(OrderShippingRule.class);
		osr.setOrder(getUser().getOrder());
		osr.setOrderItem(null);
		osr.setTotal(new Double(quantity));
		osr.setSubtotal(new Double(leftOverTotal));
		osr.setShippingRuleID(new Integer(0));
		osr.setCode("[generic]");
		osr.setRuleType("[package]");
		osr.setFloor(new Double(0.0));
		osr.setCeiling(new Double(0.0));
		osr.setPkgWeight(new Double(weight));

		if (getUser().getOrder().getOrderShippingRules() == null) {
			getUser().getOrder().setOrderShippingRules(new LinkedHashSet());
		}
		getUser().getOrder().getOrderShippingRules().add(osr);

	}

	public int createPackage(ShippingRule sr, ShippingRuleRange srr,
			OrderItem oi, int currentQuantity, boolean firstRule)
			throws Exception {
		int newQuantity = currentQuantity;

		// This simply adds one line to the order shipping rule table. It means
		// that we are adding a package with a certain number of items from 1
		// line item. For instance, a user might have 24 green shirts as one
		// line item in a basket. This will get the max number that fits into a
		// package and return what is left over.
		if (log.isDebugEnabled())
			log.debug("Adding shipping rule: " + sr.getCode());
		OrderShippingRule osr =  getBusinessObjectFactory()
				.createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(osr, sr);
		BeanUtils.copyProperties(osr, srr);
		if (osr.getCeiling().doubleValue() >= currentQuantity) {
			// Put them all in
			osr.setTotal(new Double(currentQuantity));
			newQuantity = 0;
		} else {
			osr.setTotal(osr.getCeiling());
			newQuantity = currentQuantity
					- ((int) osr.getCeiling().doubleValue());
		}

		// Figure out the total for our line in the shipping table
		if (firstRule) {
			osr.setSubtotal(new Double((oi.getTotal().doubleValue() / oi
					.getQuantity())
					* osr.getTotal().doubleValue()));
		} else {
			osr.setSubtotal(new Double(0));
		}

		// If the floor and ceiling are not equal, assume the package's portion
		// of the order item's weight should be added to the package weight
		if (firstRule
				&& srr.getCeiling().doubleValue() != srr.getFloor()
						.doubleValue()) {
			if (osr.getPkgWeight() == null) {
				osr.setPkgWeight(new Double(0));
			}
			double thisPackagesPortion = (oi.getWeight().doubleValue() / oi
					.getQuantity())
					* osr.getTotal().doubleValue();
			osr.setPkgWeight(new Double(osr.getPkgWeight().doubleValue()
					+ thisPackagesPortion));
		}

		// Set order shipping rule's relationships
		osr.setOrder(getUser().getOrder());
		osr.setOrderItem(oi);
		if (getUser().getOrder().getOrderShippingRules() == null) {
			getUser().getOrder().setOrderShippingRules(new LinkedHashSet());
		}
		getUser().getOrder().getOrderShippingRules().add(osr);
		if (oi.getOrderShippingRules() == null) {
			oi.setOrderShippingRules(new LinkedHashSet());
		}
		oi.getOrderShippingRules().add(osr);

		return newQuantity;
	}

	public void createPossibleDiscount(ShippingRule sr, ShippingRuleRange srr,
			OrderItem oi) throws Exception {
		// This only adds a "possible" discount as we don't know what method
		// they may choose
		// These are later filtered out after we know what they have done.

		if (log.isDebugEnabled())
			log.debug("Adding discount shipping rule: " + sr.getCode());
		OrderShippingRule osr =  getBusinessObjectFactory()
				.createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(osr, sr);
		BeanUtils.copyProperties(osr, srr);
		// Leave the subtotal at 0, because we don't know if it will be applied
		// yet
		osr.setSubtotal(new Double(0.0));
		osr.setOrder(getUser().getOrder());
		osr.setOrderItem(oi);
		if (getUser().getOrder().getOrderShippingRules() == null) {
			getUser().getOrder().setOrderShippingRules(new LinkedHashSet());
		}
		getUser().getOrder().getOrderShippingRules().add(osr);
		if (oi.getOrderShippingRules() == null) {
			oi.setOrderShippingRules(new LinkedHashSet());
		}
		oi.getOrderShippingRules().add(osr);
	}

	public void createMethodLimit(ShippingRule sr, ShippingRuleRange srr,
			OrderItem oi) throws Exception {
		// Add a shipping rule that only identifies a method limit
		if (log.isDebugEnabled())
			log.debug("Adding method limit shipping rule: " + sr.getCode());
		OrderShippingRule osr =  getBusinessObjectFactory()
				.createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(osr, sr);
		BeanUtils.copyProperties(osr, srr);
		osr.setOrder(getUser().getOrder());
		osr.setOrderItem(oi);
		if (getUser().getOrder().getOrderShippingRules() == null) {
			getUser().getOrder().setOrderShippingRules(new LinkedHashSet());
		}
		getUser().getOrder().getOrderShippingRules().add(osr);
		if (oi.getOrderShippingRules() == null) {
			oi.setOrderShippingRules(new LinkedHashSet());
		}
		oi.getOrderShippingRules().add(osr);
	}

	public boolean isShippingRuleOk(ShippingRule shippingRule, OrderItem oi)
			throws Exception {
		// We can do some checks here later if needed.
		// We dont have an active flag or anything yet.
		return true;
	}

	public Collection getShippingRulesBySkuIDs(Collection skuIDs)
			throws Exception {
		ShippingRuleGatewayDAO shippingRuleGatewayDAO =  getDaoFactory()
				.createDAO(ShippingRuleGatewayDAO.class);
		return shippingRuleGatewayDAO.getShippingRulesBySkuIDs(skuIDs);
	}

	public Collection getSkuIDs(OrderItem oi, Collection matchingSkus) {
		Collection skuIDs = new ArrayList();
		Iterator j = matchingSkus.iterator();
		while (j.hasNext()) {
			Map sku = (Map) j.next();
			if (oi.equals((OrderItem) sku.get("orderItem"))) {
				skuIDs.add(new Integer((String) sku.get("skuID")));
			}
		}
		return skuIDs;
	}

	public Map processRemoveAllRules() throws Exception {
		if (log.isDebugEnabled())
			log.debug("Removing all existing shipping rules for the order");

		Collection orderShippingRules = getUser().getOrder()
				.getOrderShippingRules();
		if (orderShippingRules == null) {
			return null;
		}

		Iterator i = orderShippingRules.iterator();
		while (i.hasNext()) {
			OrderShippingRule osr = (OrderShippingRule) i.next();
			if (osr.getOrderItem() != null
					&& osr.getOrderItem().getOrderShippingRules() != null) {
				osr.getOrderItem().getOrderShippingRules().remove(osr);
			}
			i.remove();
		}

		return null;
	}
}
