package com.softslate.commerce.businessobjects.shipping;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with shipping rule ranges.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting shipping rule ranges.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ShippingRuleRangeProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "shippingRuleRangeProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "shippingRuleRangeProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingRuleRangeProcessor}.
 * 
 * @author Jason McWilliams
 */
public interface ShippingRuleRangeProcessor extends BusinessProcessor {

	abstract Map loadShippingRuleRangesAndCount(Map parameters) throws Exception;

	abstract Map addShippingRuleRange(Map parameters) throws Exception;

	abstract ShippingRuleRange loadShippingRuleRangeFromID(Map parameters)
			throws Exception;

	abstract Map editShippingRuleRange(Map parameters) throws Exception;

	abstract Map editDeleteShippingRuleRanges(Map parameters) throws Exception;

	abstract void deleteShippingRuleRange(Map parameters) throws Exception;
}
