package com.softslate.commerce.customer.core;

import java.util.Locale;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;
import org.apache.struts.util.MessageResources;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.User;

/**
 * Extends the Struts <code>DynaActionForm</code> class to provide base
 * functionality for form classes in the system. Provides several properties
 * common to most of the Web requests the application receives, to help action
 * classes communicate with the rest of the application.
 * <p>
 * Within the action classes of the Struts layer, this <code>BaseDynaForm</code>
 * will be passed from method to method so that each of these properties can be
 * accessed easily.
 * 
 * @author David Tobey
 * 
 */
public class BaseDynaForm extends DynaActionForm {

    private static final long serialVersionUID = 3258417213878122288L;

    static Log log = LogFactory.getLog(BaseForm.class);

    /**
     * The current <code>HttpServletRequest</code> object pertaining to the
     * submission to this <code>BaseForm</code>.
     */
    private HttpServletRequest request = null;

    public HttpServletRequest getRequest() {
        return (this.request);
    }

    public void setRequest(HttpServletRequest request) {
        this.request = request;
    }

    /**
     * An <code>ActionMessages</code> object that can be used to store
     * messages that will be displayed to the user within the response.
     */
    private ActionMessages messages = null;

    public ActionMessages getMessages() {
        return messages;
    }

    public void setMessages(ActionMessages messages) {
        this.messages = messages;
    }

    /**
     * An <code>ActionErrors</code> object that stores any errors that will be
     * displayed to the current user within the response.
     */
    private ActionErrors errors = new ActionErrors();

    public ActionErrors getErrors() {
        return (this.errors);
    }

    public void setErrors(ActionErrors errors) {
        this.errors = errors;
    }

    /**
     * The mapping corresponding to the current action of the request.
     */
    private ActionMapping mapping = null;

    public ActionMapping getMapping() {
        return (this.mapping);
    }

    public void setMapping(ActionMapping mapping) {
        this.mapping = mapping;
    }

    /**
     * This request's settings, as loaded from the <code>sscSetting</code>
     * database table.
     */
    private Settings settings = null;

    public Settings getSettings() {
        return (this.settings);
    }

    public void setSettings(Settings settings) {
        this.settings = settings;
    }

    /**
     * The current user making the request for this <code>BaseForm</code>.
     * This object lives in the user's session across requests, but can be
     * passed throughout the Struts layer through this property.
     */
    private User user = null;

    public User getUser() {
        return (this.user);
    }

    public void setUser(User user) {
        this.user = user;
    }

    /**
     * The application's business object factory. Available here so that any of
     * the Struts actions processing the request can use it to easily
     * instantiate business objects.
     */
    private BusinessObjectFactory businessObjectFactory = null;

    public BusinessObjectFactory getBusinessObjectFactory() {
        return (this.businessObjectFactory);
    }

    public void setBusinessObjectFactory(
            BusinessObjectFactory businessObjectFactory) {
        this.businessObjectFactory = businessObjectFactory;
    }

    /**
     * Initializes all the properties of this <code>BaseForm</code> by copying
     * the objects from the request, session, and application scopes. Called by
     * action classes and within the <code>validate</code> method of this
     * <code>BaseForm</code>.
     * 
     * @param mapping
     *            The mapping corresponding to the current action of the
     *            request.
     * @param request
     *            The <code>HttpServletRequest</code> object for this request.
     */
    public void initializeProperties(ActionMapping mapping,
            HttpServletRequest request) {
        // Make the request available to all of this class's methods.
        setRequest(request);
        // Make the mapping available to all of this class's methods.
        setMapping(mapping);
        // Create a messages object for all the other methods to use.
        setMessages(new ActionMessages());
        // Create an error object for all the other methods to use.
        setErrors(new ActionErrors());
        // Load in the settings bean for required fields
        setSettings((Settings) request.getAttribute("settings"));
        // Load in the user bean from the session scope
        setUser((User) request.getSession().getAttribute("user"));
        // Load in the business object factory from the request scope
        setBusinessObjectFactory((BusinessObjectFactory) getRequest()
                .getAttribute("businessObjectFactory"));
    }

    /**
     * Look up and return a message from a given bundle in the Struts
     * <code>MessageResources</code> objects in the application scope.
     * 
     * @param module
     *            Name of the servlet context attribute for the message
     *            resources bundle.
     * @param key
     *            Message key to be looked up and returned.
     * @param args
     *            Replacement parameters for this message.
     * @return message The message, or <code>null</code> if the message can't
     *         be found.
     */
    public String getMessage(String module, String key, Object args[]) {
        // Load in the Locale and MessageResources
        String bundle = "/" + module;
        ServletContext servletContext = getServlet().getServletContext();
        MessageResources resources = (MessageResources) servletContext
                .getAttribute(bundle);
        Locale userLocale = null;
        if (getUser() != null && getUser().getLocale() != null) {
            userLocale = getUser().getLocale();
        } else {
            userLocale = Locale.getDefault();
        }
        String message = null;
        // Look up the requested message
        if (args == null) {
            message = resources.getMessage(userLocale, key);
        } else {
            message = resources.getMessage(userLocale, key, args);
        }
        if (message == null) {
            message = "";
        }
        return message;
    }

    private FormUtils formUtils = null;

    public FormUtils getFormUtils() {
        if (formUtils != null) {
            return formUtils;
        }
        formUtils = (FormUtils) getBusinessObjectFactory()
                .createUtilsObject("formUtilsImplementer");
        return formUtils;
    }

	public String html5OrXhtml() {
		return getFormUtils().html5OrXhtml(getRequest(), getSettings());
	}
}
