package com.softslate.commerce.customer.core;

import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.strutsel.taglib.tiles.ELInsertTag;

/**
 * Extension of the Tiles <code>ElInsertTag</code>, to allow for custom
 * layouts. Before including a new tile, this tag first checks to see if it
 * exists under the user's custom layout directory. (See the "Creating a Custom
 * Layout" chapter of the User Guide.)
 * 
 * @author David Tobey
 */
public class ELAppInsertTag extends ELInsertTag {
	private static final long serialVersionUID = 3258411711991461432L;

	static Log log = LogFactory.getLog(ELAppInsertTag.class);

	/**
	 * Override the regular forward to use the file in the user's custom layout,
	 * if it exists. Replaces pages matching
	 * {@link com.softslate.commerce.customer.core.AppUtils#systemLayoutDirectory}
	 * with the user's custom layout directory. If the resulting page doesn't
	 * map to an existing file, uses the original instead.
	 * <p>
	 * This same process happens in
	 * {@link com.softslate.commerce.customer.core.BaseRequestProcessor}<!--
	 * --> to handle forwards coming from the Struts request.
	 * 
	 * @param uri
	 *            The page that will be included (as defined in the Tiles
	 *            definition XML files)
	 * @throws ServletException -
	 *             Thrown by call to pageContext.include()
	 * @throws IOException -
	 *             Thrown by call to pageContext.include()
	 * 
	 * @see com.softslate.commerce.customer.core.AppUtils#alterURI(String,
	 *      String, ServletContext)
	 */
	protected void doInclude(String uri, boolean flush)
			throws ServletException, IOException {

		// Alter the URI according the user's layout.
		String currentlayout = "currentLayout";
		if (uri.indexOf("/administrator/") > 0
				|| pageContext.getSession().getAttribute(currentlayout) == null)
			currentlayout = "adminCurrentLayout";
		String layout = (String) pageContext.getSession().getAttribute(
				currentlayout);
		uri = AppUtils.alterURI(uri, layout, pageContext.getServletContext());
		pageContext.getRequest().setAttribute("sscIncludingPage", uri);
		super.doInclude(uri, flush);
	}
}
