package com.softslate.commerce.customer.core;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.emailer.businessobjects.core.EmailerProcessor;

/**
 * The Struts action class responsible for sending emails with information
 * captured from the contact form and other form-to-email forms.
 * 
 * @author David Tobey
 * 
 */
public class Form2EmailAction extends BaseAction {

	static Log log = LogFactory.getLog(Form2EmailAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		Form2EmailForm baseForm = (Form2EmailForm) form;
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");

		// Process the email
		try {

			// Get email parameters from the settings
			String toEmails = (String) baseForm.getSettings().getValue(
					baseForm.getFormType() + "EmailTo");
			String ccEmails = (String) baseForm.getSettings().getValue(
					baseForm.getFormType() + "EmailCC");
			String bccEmails = (String) baseForm.getSettings().getValue(
					baseForm.getFormType() + "EmailBCC");
			String subject = (String) baseForm.getSettings().getValue(
					baseForm.getFormType() + "EmailSubject");
			String fromEmail = (String) baseForm.getSettings().getValue(
					"storeEmailAddress");
			if (request.getParameter("Email") != null
					&& baseForm.formUtils().isValidEmail(
							request.getParameter("Email"))) {
				fromEmail = request.getParameter("Email");
			}
			String smtpServer = (String) baseForm.getSettings().getValue(
					"storeSMTPServer");
			String smtpUsername = (String) baseForm.getSettings().getValue(
					"storeSMTPUsername");
			String smtpPassword = (String) baseForm.getSettings().getValue(
					"storeSMTPPassword");
			String smtpPort = (String) baseForm.getSettings().getValue(
					"storeSMTPPort");
			String characterEncoding = (String) baseForm.getSettings()
					.getValue("characterEncoding");
			String textMessage = baseForm.constructEmailText(request);
			String htmlMessage = baseForm.constructEmailHTML(request);

			// Send the email
			actionUtils.sendMultipartEmail(toEmails.trim(), ccEmails.trim(),
					bccEmails.trim(), fromEmail.trim(), smtpServer, subject,
					textMessage, htmlMessage, smtpUsername.trim(),
					smtpPassword.trim(), smtpPort.trim(),
					characterEncoding.trim());

			// Add a message to the user
			baseForm.getMessages().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.detail",
							"Your request was sent. Thank you."));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			request.setAttribute("formSubmittedSuccessfully", "true");

		} catch (Exception e) {
			log.warn("Error sending contact email:", e);
			ActionErrors errors = new ActionErrors();
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.detail", "Error sending email: " + e.toString()));
			saveErrors(request, (ActionMessages) errors);
			return mapping.findForward("failure");
		}

		// Process the emailer integration
		try {
			Map parameters = actionUtils.buildParameterMap(request);
			parameters.putAll(PropertyUtils.describe(form));
			EmailerProcessor processor = (EmailerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"emailerProcessorImplementer");
			processor.processContactForm(parameters);
		} catch (Exception e) {
			log.warn("Error signing up user with Emailer: ", e);
		}

		return mapping.findForward("success");
	}
}
