package com.softslate.commerce.customer.core;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.product.CategoryAction;

/**
 * 
 * Struts action class that corresponds to the welcome page. Used for requests
 * to "/Welcome.do".
 * <p>
 * The <code>execute</code> method of this class first attempts to load a
 * content page whose code is '_welcome'. If the content page isn't found, for
 * backwards compatibility, it forwards the request to the core.welcome forward.
 * <p>
 * In addition, this class collapses the category tree.
 * 
 * @author David Tobey
 */
public class WelcomeAction extends CategoryAction {
	static Log log = LogFactory.getLog(WelcomeAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		// Collapse the category tree
		synchronized (AppUtils.getSessionLock(request.getSession())) {
			request.getSession().removeAttribute("currentCategory");
		}

		// Set the isWelcomePage request variable, used by JSPs
		request.setAttribute("isWelcomePage", Boolean.TRUE);

		// Look for and return the '_welcome' content page
		try {
			BaseDynaForm baseForm = (BaseDynaForm) form;
			baseForm.initializeProperties(mapping, request);
			baseForm.set("code", "_welcome");
			ActionForward af = super.execute(mapping, baseForm, request,
					response);
			if (af.getName() != null && af.getName().equals("success")) {
				return mapping.findForward("successAsCategory");
			} else if (af.getName() != null && af.getName().equals("successFullLayout")) {
				return mapping.findForward("successAsCategoryFullLayout");
			} else if (af.getName() != null && af.getName().equals("successAjax")) {
				return mapping.findForward("successAsCategoryAjax");
			}

			// Clear any errors so they are not displayed
			baseForm.getErrors().clear();
			this.saveErrors(request, baseForm.getErrors());
		} catch (Exception e) {

			// Ignore any exceptions other than to log them
			if (log.isErrorEnabled()) {
				log.error("Exception looking for _welcome content page: ", e);
			}
		}

		// If the content page wasn't found, forward to core.welcome for
		// backward compatibility (displays the welcome.message message).
		return mapping.findForward("success");

	}

}
