package com.softslate.commerce.customer.customer;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to view the file download form. Used
 * for requests to "/AccountDownload.do".
 * <p>
 * The <code>execute</code> method of this class uses
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#loadDownloadFiles()}
 * to load the file downloads that correspond to the products the customer has
 * prevously ordered. It places a <code>Collection</code> of <code>Map</code>
 * objects, one for each available download, in the request scope as an
 * attribute named "downloads".
 * 
 * @author David Tobey
 * 
 */
public class AccountDownloadAction extends BaseAction {
	static Log log = LogFactory.getLog(AccountDownloadAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {

			// Preserve the original request so the customer is redirected
			// back here after logging in
			request
					.setAttribute("originalRequest", request
							.getRequestURI());
			
			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			// Query for the customer's available downloads
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			Collection downloads = customerProcessor.loadDownloadFiles();
			// Save the downloads to the request scope
			baseForm.getRequest().setAttribute("downloads", downloads);
			return mapping.findForward("success");
		}
	}
}
