package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to view the detail of an order in a
 * customer's order history. Used for requests to "/OrderDetails.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#loadOrderDetails(Map)}
 * to load the details of a given order into the request. If the order is loaded
 * successfully, an instance of
 * {@link com.softslate.commerce.businessobjects.order.Order} will be placed in
 * the request scope, as an attribute named "order". If the order cannot be
 * found in the database, the <code>ActionForward</code> named "failure" will be
 * returned.
 * 
 * @author David Tobey
 * 
 */
public class OrderDetailsAction extends BaseAction {
	static Log log = LogFactory.getLog(OrderDetailsAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		Map parameters = PropertyUtils.describe(baseForm);
		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {

			// Preserve the original request so the customer is redirected
			// back here after logging in
			request.setAttribute("originalRequest", request.getRequestURI()
					+ "?orderNumber=" + StringEscapeUtils.escapeXml11((String) parameters.get("orderNumber")));

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			// Query for the customer's order history
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			Order order =  baseForm.getBusinessObjectFactory()
					.createObject(Order.class);
			order = customerProcessor.loadOrderDetails(parameters);
			// Query for the customer's order
			if (order == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.orderNotFound"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				return mapping.findForward("failure");
			}
			// Save the order to the request scope
			baseForm.getRequest().setAttribute("order", order);

			return mapping.findForward("success");
		}
	}
}
