package com.softslate.commerce.customer.payment.paypoint;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.payment.paypoint.IPaypointCallbackProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.order.CheckoutAction;

/**
 * Action class to handle Paypoint callbacks.
 * 
 * @author jmcwill
 */
public class PaypointCallbackAction extends CheckoutAction {
	static Log log = LogFactory.getLog(PaypointCallbackAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		BaseForm baseForm = (BaseForm) form;
		baseForm.initializeProperties(mapping, request);

		Map parameters = PropertyUtils.describe(baseForm);

		IPaypointCallbackProcessor processor = (IPaypointCallbackProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"paypointCallbackProcessorImplementer");

		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {

			// Process the results
			Map results = processor.processCallback(parameters);

			String resultCode = (String) results.get("resultCode");
			String resultMessage = (String) results.get("resultMessage");

			// Added successful payment
			if (resultCode != null && resultCode.equals("0")) {
				processOrderComplete(baseForm);
				return baseForm.getMapping()
						.findForward("successOrderComplete");
			}

			// Bad results
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", resultMessage));

			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
					.getErrors());

			return baseForm.getMapping().findForward("failurePaymentProcess");

		}
	}

}
