package com.softslate.commerce.daos.core;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.RequestLog;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscRequestLog</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class RequestLogDAOHibernate extends BaseDAO implements DAOInterface,
		RequestLogDAO {

	static Log log = LogFactory.getLog(RequestLogDAOHibernate.class);

	private RequestLog requestLog = null;

	public RequestLog getRequestLog() {
		return (this.requestLog);
	}

	public void setRequestLog(RequestLog requestLog) {
		this.requestLog = requestLog;
	}

	public void loadRequestLogFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			RequestLog record = (RequestLog) session.get(RequestLog.class,
					new Integer(getRequestLog().getRequestLogID()));
			PropertyUtils.copyProperties(getRequestLog(), record);
			setRequestLog(record);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertRequestLog() throws DataAccessException {
		insertRequestLog(false);
	}

	public void insertRequestLog(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			getRequestLog().setCreated(formatDateTime(new java.util.Date()));
			session.save(getRequestLog());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateRequestLog() throws DataAccessException {
		updateRequestLog(false);
	}

	public void updateRequestLog(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getRequestLog());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void deleteRequestLog() throws DataAccessException {
		deleteRequestLog(false);
	}

	public void deleteRequestLog(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setRequestLog((RequestLog) session.load(RequestLog.class,
					new Integer(getRequestLog().getRequestLogID())));
			session.delete(getRequestLog());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}
}
