package com.softslate.commerce.daos.order;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Interface for a DAO to manage the <code>sscOrder</code> table of the
 * database.
 * 
 * @author David Tobey
 */
public interface OrderDAO extends DAOInterface {
	abstract Order getOrder();

	abstract void setOrder(Order order);

	abstract int getMaxOrderNumber() throws Exception;

	abstract void insertOrder() throws Exception;

	abstract void insertOrder(boolean commit) throws Exception;

	abstract void updateOrder() throws Exception;

	abstract void updateOrder(boolean commit) throws Exception;

	/**
	 * Assuming the orderNumber and customerID properties have been set, load
	 * the <code>Order</code> object from the database.
	 */
	abstract void loadFromNumberAndCustomerID() throws Exception;

	/**
	 * Assuming the orderID and customerID properties have been set, load the
	 * <code>Order</code> object from the database. Used to pick up saved
	 * carts.
	 */
	abstract void loadSavedCart() throws Exception;

	abstract void loadSavedCartFromToken() throws Exception;

	/**
	 * Assuming the orderID and customerID properties have been set, load the
	 * <code>Order</code> object from the database. Used to pick up reordered
	 * orders.
	 */
	abstract void loadPreviousOrder() throws Exception;

	abstract void loadOrderFromNumber() throws Exception;

	abstract Order loadOrderFromID(int orderID) throws Exception;

	abstract void loadOrderFromID() throws Exception;

	abstract void deleteOrder() throws Exception;

	abstract void deleteOrder(boolean commit) throws Exception;

	abstract void initializeOrder(Order order);

	abstract Collection loadSettings() throws DataAccessException;

	abstract Collection processSettings(Map parameters) throws Exception;

	abstract Collection processSettings(boolean commit, Map parameters)
			throws Exception;
}
