package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.OrderDiscount;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscOrderDiscount</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class OrderDiscountDAOHibernate extends BaseDAO implements DAOInterface,
        OrderDiscountDAO {

    static Log log = LogFactory.getLog(OrderDiscountDAOHibernate.class);

    private OrderDiscount orderDiscount = null;

    public OrderDiscount getOrderDiscount() {
        return (this.orderDiscount);
    }

    public void setOrderDiscount(OrderDiscount orderDiscount) {
        this.orderDiscount = orderDiscount;
    }

    public void loadOrderDiscountFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            OrderDiscount record = (OrderDiscount) session.get(
                    OrderDiscount.class, new Integer(getOrderDiscount()
                            .getOrderDiscountID()));
            PropertyUtils.copyProperties(getOrderDiscount(), record);
            setOrderDiscount(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOrderDiscount() throws DataAccessException {
        insertOrderDiscount(false);
    }

    public void insertOrderDiscount(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getOrderDiscount());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOrderDiscount() throws DataAccessException {
        updateOrderDiscount(false);
    }

    public void updateOrderDiscount(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getOrderDiscount());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOrderDiscount() throws DataAccessException {
        deleteOrderDiscount(false);
    }

    public void deleteOrderDiscount(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setOrderDiscount((OrderDiscount) session.load(OrderDiscount.class,
                    new Integer(getOrderDiscount().getOrderDiscountID())));
            session.delete(getOrderDiscount());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
