package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscProduct</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class ProductDAOHibernate extends BaseDAO implements ProductDAO {

	static Log log = LogFactory.getLog(ProductDAOHibernate.class);

	private Product product = null;

	public Product getProduct() {
		return (this.product);
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	public void loadProduct() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading product: " + getProduct().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectFromCode");
			q.setString("code", getProduct().getCode());
			q.setCacheable(true);
			Product loadedProduct = (Product) q.uniqueResult();
			setProduct(loadedProduct);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProductFromSeoCode() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading product from SEO code: "
					+ getProduct().getSeoCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectFromSeoCode");
			q.setString("seoCode", getProduct().getSeoCode());
			q.setCacheable(true);
			Product loadedProduct = (Product) q.uniqueResult();
			setProduct(loadedProduct);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProductFromCode() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading product: " + getProduct().getCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectProductFromCode");
			q.setString("code", getProduct().getCode());
			Product loadedProduct = (Product) q.uniqueResult();
			setProduct(loadedProduct);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProductFromSeoCodeAdmin() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading product from SEO code: "
					+ getProduct().getSeoCode());
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectFromSeoCodeAdmin");
			q.setString("seoCode", getProduct().getSeoCode());
			q.setCacheable(true);
			Product loadedProduct = (Product) q.uniqueResult();
			setProduct(loadedProduct);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadProductFromID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Product product = (Product) session.get(Product.class, new Integer(
					getProduct().getProductID()));
			if (product != null) {
				PropertyUtils.copyProperties(getProduct(), product);
			}
			setProduct(product);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void loadAttributesAndOptions() throws Exception {
		Session session = getDaoFactory().startSession();
		Collection results = null;
		try {
			// Load just the attributes. The options will be loaded lazily
			Query q = getNamedQuery(session, "product.loadAttributes");
			q.setInteger("productID", getProduct().getProductID());
			results = new LinkedHashSet(q.list());
			getProduct().setAttributesAndOptions(results);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void insertProduct() throws DataAccessException {
		insertProduct(false);
	}

	public void insertProduct(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		getProduct().setCreated(formatDateTime(new Date()));//timestamp the creation of this entity record.
		try {
			session.save(getProduct());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateProduct() throws DataAccessException {
		updateProduct(false);
	}

	public void updateProduct(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.update(getProduct());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
		} catch (HibernateException he) {
			log.error("Hibernate error: " + he.toString(), he);
			throw new DataAccessException(he);
		}
		return;
	}
    
	public void updateProductOrderByID(int productID, int productOrder) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Query q = getNamedQuery(session, "product.updateProductOrderByID");
            q.setInteger("productID", productID);
            q.setInteger("productOrder", productOrder);
            q.executeUpdate();
			evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
        }
        return;
	}

	public void updateProductByCode() throws Exception {
		updateProductByCode(false);
	}

	public void updateProductByCode(boolean commit) throws Exception {
		Map product = PropertyUtils.describe(getProduct());
		loadProductFromCode();
		PropertyUtils.copyProperties(getProduct(), product);
		updateProduct();
	}

	public void deleteProduct() throws DataAccessException {
		deleteProduct(false);
	}

	public void deleteProduct(boolean commit) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			setProduct((Product) session.load(Product.class, new Integer(
					getProduct().getProductID())));
			session.delete(getProduct());
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Collection loadSettings() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		Collection results = null;
		try {
			Query q = getNamedQuery(session, "productSetting.selectAll");
			q.setInteger("productID", getProduct().getProductID());
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return results;
	}

	public Object loadObject(Object object) throws Exception {
		if (null != object && object instanceof Product) {
			this.setProduct((Product) object);
			if (0 != this.getProduct().getProductID()) {
				this.loadProductFromID();
			} else if (null != this.getProduct().getCode()) {
				this.loadProductFromCode();
			}
			return this.getProduct();
		}
		return null;
	}

	public boolean deleteObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Product) {
			this.setProduct((Product) object);
			this.deleteProduct(commit);
			return true;
		}
		return false;
	}

	public Object insertObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Product) {
			this.setProduct((Product) object);
			this.insertProduct(commit);
			return this.getProduct();
		}
		return null;
	}

	public Object updateObject(Object object, boolean commit) throws Exception {
		if (null != object && object instanceof Product) {
			this.setProduct((Product) object);
			this.updateProduct(commit);
			return this.getProduct();
		}
		return null;

	}

	public Map editProductAssignment(Product product, Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editAssignment(product, parameters, bof);
		evictCache();
		return result;
	}

}
