package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to productSettings.
 * 
 * @author David Tobey
 * 
 */
public class ProductSettingGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements ProductSettingGatewayDAO {

	static Log log = LogFactory.getLog(ProductSettingGatewayDAOHibernate.class);

	public Map loadProductSettingsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteProductSettings(Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Collection loadAllProductSettings() throws DataAccessException {
		Collection productSettings = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.selectAllProductSettings");
			productSettings = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return productSettings;
	}

	public int loadMaxProductSettingID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.maxProductSettingID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllProductSettingsInRange(int firstID, int lastID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading productSettings within an id range: " + firstID + "-" + lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}
}
